@@ -2,37 +2,37 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* globals getMessagePaneBrowser, gFolderDisplay, addMenuItem, getSelectedCalendar
+/* globals getMessagePaneBrowser, addMenuItem, getSelectedCalendar
    createEventWithDialog*/
 
-var { Extractor } = ChromeUtils.import("resource:///modules/calendar/calExtract.jsm");
-var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
-var { MailServices } = ChromeUtils.import("resource:///modules/MailServices.jsm");
+var { Extractor } = ChromeUtils.importESModule("resource:///modules/calendar/calExtract.sys.mjs");
+var { cal } = ChromeUtils.importESModule("resource:///modules/calendar/calUtils.sys.mjs");
+var { MailServices } = ChromeUtils.importESModule("resource:///modules/MailServices.sys.mjs");
 var { XPCOMUtils } = ChromeUtils.importESModule("resource://gre/modules/XPCOMUtils.sys.mjs");
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  CalEvent: "resource:///modules/CalEvent.jsm",
-  CalTodo: "resource:///modules/CalTodo.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  CalEvent: "resource:///modules/CalEvent.sys.mjs",
+  CalTodo: "resource:///modules/CalTodo.sys.mjs",
 });
 
-XPCOMUtils.defineLazyGetter(this, "extractService", () => {
-  const { CalExtractParserService } = ChromeUtils.import(
-    "resource:///modules/calendar/extract/CalExtractParserService.jsm"
+ChromeUtils.defineLazyGetter(this, "extractService", () => {
+  const { CalExtractParserService } = ChromeUtils.importESModule(
+    "resource:///modules/calendar/extract/CalExtractParserService.sys.mjs"
   );
   return new CalExtractParserService();
 });
 
 var calendarExtract = {
   onShowLocaleMenu(target) {
-    let localeList = document.getElementById(target.id);
-    let langs = [];
-    let chrome = Cc["@mozilla.org/chrome/chrome-registry;1"]
+    const localeList = document.getElementById(target.id);
+    const langs = [];
+    const chrome = Cc["@mozilla.org/chrome/chrome-registry;1"]
       .getService(Ci.nsIXULChromeRegistry)
       .QueryInterface(Ci.nsIToolkitChromeRegistry);
-    let langRegex = /^(([^-]+)-*(.*))$/;
+    const langRegex = /^(([^-]+)-*(.*))$/;
 
-    for (let locale of chrome.getLocalesForPackage("calendar")) {
-      let localeParts = langRegex.exec(locale);
+    for (const locale of chrome.getLocalesForPackage("calendar")) {
+      const localeParts = langRegex.exec(locale);
       let langName = localeParts[2];
 
       try {
@@ -41,21 +41,24 @@ var calendarExtract = {
         // If no language name is found that is ok, keep the technical term
       }
 
-      let label = cal.l10n.getCalString("extractUsing", [langName]);
+      let label = calendarExtract.l10n.formatValueSync("extract-using", { languageName: langName });
       if (localeParts[3] != "") {
-        label = cal.l10n.getCalString("extractUsingRegion", [langName, localeParts[3]]);
+        label = calendarExtract.l10n.formatValueSync("extract-using-region", {
+          languageName: langName,
+          region: localeParts[3],
+        });
       }
 
       langs.push([label, localeParts[1]]);
     }
 
     // sort
-    let pref = "calendar.patterns.last.used.languages";
-    let lastUsedLangs = Services.prefs.getStringPref(pref, "");
+    const pref = "calendar.patterns.last.used.languages";
+    const lastUsedLangs = Services.prefs.getStringPref(pref, "");
 
     langs.sort((a, b) => {
-      let idx_a = lastUsedLangs.indexOf(a[1]);
-      let idx_b = lastUsedLangs.indexOf(b[1]);
+      const idx_a = lastUsedLangs.indexOf(a[1]);
+      const idx_b = lastUsedLangs.indexOf(b[1]);
 
       if (idx_a == -1 && idx_b == -1) {
         return a[0].localeCompare(b[0]);
@@ -70,32 +73,27 @@ var calendarExtract = {
       localeList.lastChild.remove();
     }
 
-    for (let lang of langs) {
+    for (const lang of langs) {
       addMenuItem(localeList, lang[0], lang[1], null);
     }
   },
 
   extractWithLocale(event, isEvent) {
     event.stopPropagation();
-    let locale = event.target.value;
-    this.extractFromEmail(isEvent, true, locale);
+    const locale = event.target.value;
+    this.extractFromEmail(null, isEvent, true, locale);
   },
 
-  async extractFromEmail(isEvent, fixedLang, fixedLocale) {
-    // Get the message from the 3pane, or from the standalone message window.
-    let aboutMessage =
-      document.getElementById("tabmail")?.currentAboutMessage ||
-      document.getElementById("messageBrowser").contentWindow;
-    let message = aboutMessage.gMessage;
-    let folder = message.folder;
-    let title = message.mime2DecodedSubject;
+  async extractFromEmail(message, isEvent, fixedLang, fixedLocale) {
+    const folder = message.folder;
+    const title = message.mime2DecodedSubject;
 
     let content = "";
     await new Promise((resolve, reject) => {
-      let listener = {
+      const listener = {
         QueryInterface: ChromeUtils.generateQI(["nsIStreamListener"]),
         onDataAvailable(request, inputStream, offset, count) {
-          let text = folder.getMsgTextFromStream(
+          const text = folder.getMsgTextFromStream(
             inputStream,
             message.charset,
             count, // bytesToRead
@@ -107,7 +105,7 @@ var calendarExtract = {
           // If we ever got text, we're good. Ignore further chunks.
           content ||= text;
         },
-        onStartRequest(request) {},
+        onStartRequest() {},
         onStopRequest(request, statusCode) {
           if (!Components.isSuccessCode(statusCode)) {
             reject(new Error(statusCode));
@@ -115,33 +113,36 @@ var calendarExtract = {
           resolve();
         },
       };
-      let uri = message.folder.getUriForMsg(message);
+      const uri = message.folder.getUriForMsg(message);
       MailServices.messageServiceFromURI(uri).streamMessage(uri, listener, null, null, false, "");
     });
 
     cal.LOG("[calExtract] Original email content: \n" + title + "\r\n" + content);
-    let date = new Date(message.date / 1000);
-    let time = new Date().getTime();
+    const date = new Date(message.date / 1000);
+    const time = new Date().getTime();
 
-    let item;
-    item = isEvent ? new CalEvent() : new CalTodo();
+    const item = isEvent ? new CalEvent() : new CalTodo();
     item.title = message.mime2DecodedSubject;
     item.calendar = getSelectedCalendar();
-    item.setProperty("DESCRIPTION", content);
+    item.descriptionText = content;
     item.setProperty("URL", `mid:${message.messageId}`);
     cal.dtz.setDefaultStartEndHour(item);
     cal.alarms.setDefaultValues(item);
-    let sel = aboutMessage.getMessagePaneBrowser().contentWindow.getSelection();
-    // Thunderbird Conversations might be installed
-    if (sel === null) {
+    const tabmail = document.getElementById("tabmail");
+    const messagePaneBrowser =
+      tabmail?.currentTabInfo.chromeBrowser.contentWindow.messagePane.visibleMessagePaneBrowser?.() ||
+      tabmail?.currentAboutMessage?.getMessagePaneBrowser() ||
+      document.getElementById("messageBrowser")?.contentWindow?.getMessagePaneBrowser();
+    let sel = messagePaneBrowser?.contentWindow?.getSelection();
+    // Check if there's an iframe with a selection (e.g. Thunderbird Conversations)
+    if (sel && sel.type !== "Range") {
       try {
-        sel = document
-          .getElementById("multimessage")
-          .contentDocument.querySelector(".iframe-container iframe")
+        sel = messagePaneBrowser?.contentDocument
+          .querySelector("iframe")
           .contentDocument.getSelection();
       } catch (ex) {
         // If Thunderbird Conversations is not installed that is fine,
-        // we will just have a null selection.
+        // we will just have an empty or null selection.
       }
     }
 
@@ -151,7 +152,7 @@ var calendarExtract = {
     let collected = [];
     let useService = Services.prefs.getBoolPref("calendar.extract.service.enabled");
     if (useService) {
-      let result = extractService.extract(content, { now: date });
+      const result = extractService.extract(content, { now: date });
       if (!result) {
         useService = false;
       } else {
@@ -161,8 +162,8 @@ var calendarExtract = {
     }
 
     if (!useService) {
-      let locale = Services.locale.requestedLocale;
-      let dayStart = Services.prefs.getIntPref("calendar.view.daystarthour", 6);
+      const locale = Services.locale.requestedLocale;
+      const dayStart = Services.prefs.getIntPref("calendar.view.daystarthour", 6);
       if (fixedLang) {
         extractor = new Extractor(fixedLocale, dayStart);
       } else {
@@ -180,7 +181,7 @@ var calendarExtract = {
         guessed = extractor.guessStart(!isEvent);
         endGuess = extractor.guessEnd(guessed, !isEvent);
       }
-      let allDay = (guessed.hour == null || guessed.minute == null) && isEvent;
+      const allDay = (guessed.hour == null || guessed.minute == null) && isEvent;
 
       if (isEvent) {
         if (guessed.year != null) {
@@ -221,8 +222,8 @@ var calendarExtract = {
           item.endDate.minute = endGuess.minute;
         }
       } else {
-        let dtz = cal.dtz.defaultTimezone;
-        let dueDate = new Date();
+        const dtz = cal.dtz.defaultTimezone;
+        const dueDate = new Date();
         // set default
         dueDate.setHours(0);
         dueDate.setMinutes(0);
@@ -258,67 +259,16 @@ var calendarExtract = {
       }
     }
 
-    let timeSpent = new Date().getTime() - time;
+    const timeSpent = new Date().getTime() - time;
     cal.LOG(
       "[calExtract] Total time spent for conversion (including loading of dictionaries): " +
         timeSpent +
         "ms"
     );
   },
-
-  addListeners() {
-    if (window.top.document.location == "chrome://messenger/content/messenger.xhtml") {
-      // covers initial load and folder change
-      let folderTree = document.getElementById("folderTree");
-      folderTree.addEventListener("select", this.setState);
-
-      // covers selection change in a folder
-      let msgTree = window.top.GetThreadTree();
-      msgTree.addEventListener("select", this.setState);
-
-      window.addEventListener("unload", () => {
-        folderTree.removeEventListener("select", this.setState);
-        msgTree.removeEventListener("select", this.setState);
-      });
-    }
-  },
-
-  setState() {
-    let eventButton = document.getElementById("extractEventButton");
-    let taskButton = document.getElementById("extractTaskButton");
-    let contextMenu = document.getElementById("mailContext-calendar-convert-menu");
-    let contextMenuEvent = document.getElementById("mailContext-calendar-convert-event-menuitem");
-    let contextMenuTask = document.getElementById("mailContext-calendar-convert-task-menuitem");
-    let eventDisabled = gFolderDisplay.selectedCount == 0;
-    let taskDisabled = gFolderDisplay.selectedCount == 0;
-    let contextEventDisabled = false;
-    let contextTaskDisabled = false;
-    let newEvent = document.getElementById("calendar_new_event_command");
-    let newTask = document.getElementById("calendar_new_todo_command");
-
-    if (newEvent.getAttribute("disabled") == "true") {
-      eventDisabled = true;
-      contextEventDisabled = true;
-    }
-
-    if (newTask.getAttribute("disabled") == "true") {
-      taskDisabled = true;
-      contextTaskDisabled = true;
-    }
-
-    if (eventButton) {
-      eventButton.disabled = eventDisabled;
-    }
-    if (taskButton) {
-      taskButton.disabled = taskDisabled;
-    }
-
-    contextMenuEvent.disabled = contextEventDisabled;
-    contextMenuTask.disabled = contextTaskDisabled;
-
-    contextMenu.disabled = contextEventDisabled && contextTaskDisabled;
-  },
 };
-
-// TODO: Fix and reinstate this.
-// window.addEventListener("load", calendarExtract.addListeners.bind(calendarExtract));
+ChromeUtils.defineLazyGetter(
+  calendarExtract,
+  "l10n",
+  () => new Localization(["calendar/calendar.ftl"], true)
+);
