@@ -2,6 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
 
+const { setTimeout } = ChromeUtils.importESModule(
+  "resource://gre/modules/Timer.sys.mjs"
+);
+
 async function subtest() {
   // Put some cards on the server.
   CardDAVServer.putCardInternal(
@@ -182,21 +186,32 @@ async function subtest() {
 
   try {
     let changeMeCard = cardMap.get("change-me");
+    Assert.equal(
+      changeMeCard.getProperty("PopularityIndex", ""),
+      0,
+      "sanity check, initial PopularityIndex"
+    );
     changeMeCard.displayName = "I've been changed again!";
+    changeMeCard.setProperty("PopularityIndex", 10);
+
+    // First entry into AddrBookDirectory.modifyCard.
+    const firstNotification = observer.waitFor("addrbook-contact-updated");
     directory.modifyCard(changeMeCard);
     Assert.ok(!directory.readOnly, "read-only directory should throw.");
+    await firstNotification;
 
+    // Second entry into AddrBookDirectory.modifyCard. Triggered by syncing.
     Assert.equal(
       await observer.waitFor("addrbook-contact-updated"),
       "change-me"
     );
-    observer.checkAndClearNotifications({
-      "addrbook-contact-created": [],
-      "addrbook-contact-updated": ["change-me"],
-      "addrbook-contact-deleted": [],
-    });
 
     changeMeCard = directory.childCards.find(c => c.UID == "change-me");
+    Assert.equal(
+      changeMeCard.getProperty("PopularityIndex", ""),
+      10,
+      "PopularityIndex survived sync"
+    );
     cardMap.set("change-me", changeMeCard);
 
     await checkCardsOnServer({
@@ -215,6 +230,45 @@ async function subtest() {
     Assert.ok(directory.readOnly, "read-write directory should not throw");
   }
 
+  // Change a card on the client, but this time only non-vCard properties.
+
+  info("Changing a card on the client, non-vCard properties.");
+
+  try {
+    let changeMeCard = cardMap.get("change-me");
+    Assert.equal(
+      changeMeCard.getProperty("PopularityIndex", ""),
+      10,
+      "sanity check, initial PopularityIndex"
+    );
+    changeMeCard.setProperty("PopularityIndex", 20);
+
+    // First entry into AddrBookDirectory.modifyCard.
+    const firstNotification = observer.waitFor("addrbook-contact-updated");
+    directory.modifyCard(changeMeCard);
+    Assert.ok(!directory.readOnly, "read-only directory should throw");
+    await firstNotification;
+
+    // Check there's no second entry into AddrBookDirectory.modifyCard.
+    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+    await new Promise(resolve => setTimeout(resolve, 500));
+    observer.checkAndClearNotifications({
+      "addrbook-contact-created": [],
+      "addrbook-contact-updated": [],
+      "addrbook-contact-deleted": [],
+    });
+
+    changeMeCard = directory.childCards.find(c => c.UID == "change-me");
+    Assert.equal(
+      changeMeCard.getProperty("PopularityIndex", ""),
+      20,
+      "PopularityIndex did get modified"
+    );
+    cardMap.set("change-me", changeMeCard);
+  } catch (ex) {
+    Assert.ok(directory.readOnly, "read-write directory should not throw");
+  }
+
   // Add a new card on the client.
 
   info("Adding a new card on the client.");
@@ -224,6 +278,7 @@ async function subtest() {
       Ci.nsIAbCard
     );
     newCard.displayName = "I'm another new contact. ϔ";
+    newCard.setProperty("PopularityIndex", 10);
     newCard.UID = "another-new";
     newCard = directory.addCard(newCard);
     Assert.ok(!directory.readOnly, "read-only directory should throw.");
@@ -244,6 +299,11 @@ async function subtest() {
       "I'm another new contact. ϔ",
       "non-ascii character survived the trip to the server"
     );
+    Assert.equal(
+      newCard.getProperty("PopularityIndex", ""),
+      10,
+      "PopularityIndex survived sync"
+    );
 
     await checkCardsOnServer({
       "another-new": {
