@@ -8,40 +8,20 @@
  */
 
 function checkLists(childNodes, number) {
-  var mailListArray = new Array(number);
-
-  for (var i = 0; i < number; ++i) {
-    mailListArray[i] = null;
-  }
-
+  let count = 0;
   // See comment above for matching requirements
-  while (childNodes.hasMoreElements()) {
-    var list = childNodes.getNext();
-    if (
-      list instanceof Ci.nsIAbDirectory &&
-      list.isMailList &&
-      list.dirName.startsWith("TestList")
-    ) {
-      var index = list.dirName.substr(8, list.dirName.length - 8);
-      Assert.equal(mailListArray[index - 1], null);
-      Assert.equal(list.URI, kPABData.URI + "/MailList" + index);
-
-      mailListArray[index - 1] = list;
+  for (const list of childNodes) {
+    if (list.isMailList && list.dirName.startsWith("TestList")) {
+      Assert.equal(list.URI, `${kPABData.URI}/${list.UID}`);
+      count++;
     }
   }
 
-  mailListArray.forEach(function(value) {
-    Assert.notEqual(value, null);
-  });
+  Assert.equal(count, number);
 }
 
 function run_test() {
-  // Create a new card
-  // Test setup - copy the data file into place
-  var testAB = do_get_file("../../../data/abLists1.mab");
-
-  // Copy the file to the profile directory for a PAB
-  testAB.copyTo(do_get_profile(), kPABData.fileName);
+  loadABFile("../../../data/abLists1", kPABData.fileName);
 
   // Test - Get the directory.
 
@@ -49,7 +29,7 @@ function run_test() {
   // lists need help initialising themselves
   MailServices.ab.directories;
 
-  let AB = MailServices.ab.getDirectory(kPABData.URI);
+  const AB = MailServices.ab.getDirectory(kPABData.URI);
 
   // Test - Check all the expected mailing lists exist.
 
@@ -74,7 +54,9 @@ function run_test() {
 
   // Test - Remove a list.
 
-  mailList = MailServices.ab.getDirectory(kPABData.URI + "/MailList4");
+  mailList = MailServices.ab.getDirectory(
+    kPABData.URI + "/46cf4cbf-5945-43e4-a822-30c2f2969db9"
+  );
 
   AB.deleteDirectory(mailList);
 
