@@ -3,17 +3,31 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* import-globals-from retention.js */
+/* global BigInt */
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { Gloda } = ChromeUtils.import("resource:///modules/gloda/gloda.js");
+ChromeUtils.defineESModuleGetters(this, {
+  FolderTreeProperties: "resource:///modules/FolderTreeProperties.sys.mjs",
+  Gloda: "resource:///modules/gloda/Gloda.sys.mjs",
+  openLinkExternally: "resource:///modules/LinkHelper.sys.mjs",
+  UIFontSize: "resource:///modules/UIFontSize.sys.mjs",
+  MorkParser: "resource:///modules/MorkParser.sys.mjs",
+});
 
 var gMsgFolder;
 var gLockedPref = null;
 
+var gDefaultColor = "";
+
+window.addEventListener("load", folderPropsOnLoad);
 document.addEventListener("dialogaccept", folderPropsOKButton);
+document.addEventListener("dialogcancel", folderCancelButton);
 
-// The folderPropsSink is the class that gets notified of an imap folder's properties
-
+/**
+ * The folderPropsSink is the class that gets notified of an imap folder's
+ * properties.
+ *
+ * @implements {nsIMsgImapFolderProps}
+ */
 var gFolderPropsSink = {
   setFolderType(folderTypeString) {
     var typeLabel = document.getElementById("folderType.text");
@@ -64,70 +78,56 @@ var gFolderPropsSink = {
     }
   },
 
-  setQuotaData(root, usedKB, maxKB) {
-    var quotaRoot = document.getElementById("quotaRoot");
-    if (quotaRoot) {
-      quotaRoot.setAttribute("value", '"' + root + '"');
-    }
+  setQuotaData(folderQuota) {
+    const quotaDetails = document.getElementById("quotaDetails");
+    const messenger = Cc["@mozilla.org/messenger;1"].createInstance(
+      Ci.nsIMessenger
+    );
 
-    var percentage = maxKB != 0 ? Math.round((usedKB / maxKB) * 100) : 0;
+    for (const quota of folderQuota) {
+      const li = document.createElement("li");
+      const name = document.createElement("span");
+      name.textContent = quota.name;
+      li.appendChild(name);
 
-    var quotaPercentageBar = document.getElementById("quotaPercentageBar");
-    if (quotaPercentageBar) {
-      quotaPercentageBar.setAttribute("value", percentage);
-    }
+      const progress = document.createElement("progress");
+      progress.classList.add("quota-percentage");
+      progress.setAttribute("value", quota.usage);
+      progress.setAttribute("max", quota.limit);
 
-    var bundle = document.getElementById("bundle_messenger");
-    if (bundle) {
-      var usedFreeCaption = bundle.getFormattedString(
-        "quotaUsedFree",
-        [usedKB, maxKB],
-        2
-      );
-      var quotaCaption = document.getElementById("quotaUsedFree");
-      if (quotaCaption) {
-        quotaCaption.setAttribute("value", usedFreeCaption);
+      li.appendChild(progress);
+
+      const percentage = document.createElement("span");
+      document.l10n.setAttributes(percentage, "quota-percent-used", {
+        percent: Number((100n * BigInt(quota.usage)) / BigInt(quota.limit)),
+      });
+      li.appendChild(percentage);
+
+      li.appendChild(document.createTextNode(" — "));
+
+      const details = document.createElement("span");
+      if (/STORAGE/i.test(quota.name)) {
+        const usage = messenger.formatFileSize(quota.usage * 1024);
+        const limit = messenger.formatFileSize(quota.limit * 1024);
+        details.textContent = `${usage} / ${limit}`;
+      } else {
+        details.textContent = `${quota.usage} / ${quota.limit}`;
       }
+      li.appendChild(details);
 
-      var percentUsedCaption = bundle.getFormattedString(
-        "quotaPercentUsed",
-        [percentage],
-        1
-      );
-      var percentUsed = document.getElementById("quotaPercentUsed");
-      if (percentUsed) {
-        percentUsed.setAttribute("value", percentUsedCaption);
-      }
+      quotaDetails.appendChild(li);
     }
   },
 };
 
 function doEnabling() {
   var nameTextbox = document.getElementById("name");
-  document.documentElement.getButton("accept").disabled = !nameTextbox.value;
+  document.querySelector("dialog").getButton("accept").disabled =
+    !nameTextbox.value;
 }
 
 function folderPropsOKButton(event) {
   if (gMsgFolder) {
-    // set charset attributes
-    var folderCharsetList = document.getElementById("folderCharsetList");
-
-    // Log to the Error Console the charset value for the folder
-    // if it is unknown to us. Value will be preserved by the menu-item.
-    if (folderCharsetList.selectedIndex == -1) {
-      Cu.reportError(
-        "Unknown folder encoding; folder=" +
-          gMsgFolder.name +
-          ", charset=" +
-          gMsgFolder.charset
-      );
-    }
-
-    gMsgFolder.charset = folderCharsetList.getAttribute("value");
-    gMsgFolder.charsetOverride = document.getElementById(
-      "folderCharsetOverride"
-    ).checked;
-
     if (
       document.getElementById("offline.selectForOfflineFolder").checked ||
       document.getElementById("offline.selectForOfflineNewsgroup").checked
@@ -143,7 +143,9 @@ function folderPropsOKButton(event) {
       gMsgFolder.clearFlag(Ci.nsMsgFolderFlags.CheckNew);
     }
 
-    let glodaCheckbox = document.getElementById("folderIncludeInGlobalSearch");
+    const glodaCheckbox = document.getElementById(
+      "folderIncludeInGlobalSearch"
+    );
     if (!glodaCheckbox.hidden) {
       if (glodaCheckbox.checked) {
         // We pass true here so that folders such as trash and junk can still
@@ -164,21 +166,50 @@ function folderPropsOKButton(event) {
       "retention.useDefault"
     ).checked;
     gMsgFolder.retentionSettings = retentionSettings;
+
+    let color = document.getElementById("color").value;
+    if (color == gDefaultColor) {
+      color = undefined;
+    }
+    FolderTreeProperties.setColor(gMsgFolder.URI, color);
+    // Tell 3-pane tabs to update the folder's color.
+    Services.obs.notifyObservers(gMsgFolder, "folder-color-changed", color);
   }
 
   try {
     // This throws an exception when an illegal folder name was entered.
     top.okCallback(
       document.getElementById("name").value,
-      window.arguments[0].name,
-      gMsgFolder.URI
+      window.arguments[0].name
     );
   } catch (e) {
     event.preventDefault();
   }
 }
 
+function folderCancelButton() {
+  // Clear any previewed color.
+  Services.obs.notifyObservers(gMsgFolder, "folder-color-preview");
+}
+
 function folderPropsOnLoad() {
+  UIFontSize.registerWindow(window);
+  const styles = getComputedStyle(document.body);
+  const folderColors = {
+    Inbox: styles.getPropertyValue("--folder-color-inbox"),
+    Sent: styles.getPropertyValue("--folder-color-sent"),
+    Outbox: styles.getPropertyValue("--folder-color-outbox"),
+    Drafts: styles.getPropertyValue("--folder-color-draft"),
+    Trash: styles.getPropertyValue("--folder-color-trash"),
+    Archive: styles.getPropertyValue("--folder-color-archive"),
+    Templates: styles.getPropertyValue("--folder-color-template"),
+    Spam: styles.getPropertyValue("--folder-color-spam"),
+    Virtual: styles.getPropertyValue("--folder-color-folder-filter"),
+    RSS: styles.getPropertyValue("--folder-color-rss"),
+    Newsgroup: styles.getPropertyValue("--folder-color-newsletter"),
+  };
+  gDefaultColor = styles.getPropertyValue("--folder-color-folder");
+
   // look in arguments[0] for parameters
   if (window.arguments && window.arguments[0]) {
     if (window.arguments[0].title) {
@@ -189,11 +220,9 @@ function folderPropsOnLoad() {
     }
   }
 
-  // fill in folder name, based on what they selected in the folder pane
   if (window.arguments[0].folder) {
+    // Fill in folder name, based on what they selected in the folder pane.
     gMsgFolder = window.arguments[0].folder;
-  } else {
-    dump("passed null for folder, do nothing\n");
   }
 
   if (window.arguments[0].name) {
@@ -202,9 +231,6 @@ function folderPropsOnLoad() {
     // when the dialog is accepted.
     var nameTextbox = document.getElementById("name");
     nameTextbox.value = window.arguments[0].name;
-
-    // name.setSelectionRange(0,-1);
-    // name.focusTextField();
   }
 
   const serverType = window.arguments[0].serverType;
@@ -217,19 +243,61 @@ function folderPropsOnLoad() {
     // We really need a functioning database, so we'll detect problems
     // and create one if we have to.
     try {
-      gMsgFolder.getDatabase(null);
+      gMsgFolder.msgDatabase;
     } catch (e) {
       gMsgFolder.updateFolder(window.arguments[0].msgWindow);
     }
 
+    // Check the current folder name against known folder names to set the
+    // correct default color, if needed.
+    let selectedFolderName = "";
+
+    switch (window.arguments[0].serverType) {
+      case "rss":
+        selectedFolderName = "RSS";
+        break;
+      case "nntp":
+        selectedFolderName = "Newsgroup";
+        break;
+      default:
+        selectedFolderName = window.arguments[0].name;
+        break;
+    }
+
+    if (Object.keys(folderColors).includes(selectedFolderName)) {
+      gDefaultColor = folderColors[selectedFolderName];
+    }
+
+    const colorInput = document.getElementById("color");
+    colorInput.value =
+      FolderTreeProperties.getColor(gMsgFolder.URI) || gDefaultColor;
+    colorInput.addEventListener("input", () => {
+      // Preview the chosen color.
+      Services.obs.notifyObservers(
+        gMsgFolder,
+        "folder-color-preview",
+        colorInput.value
+      );
+    });
+    const resetColorButton = document.getElementById("resetColor");
+    resetColorButton.addEventListener("click", function () {
+      colorInput.value = gDefaultColor;
+      // Preview the default color.
+      Services.obs.notifyObservers(
+        gMsgFolder,
+        "folder-color-preview",
+        gDefaultColor
+      );
+    });
+
     var locationTextbox = document.getElementById("location");
 
     // Decode the displayed mailbox:// URL as it's useful primarily for debugging,
     // whereas imap and news urls are sent around.
     locationTextbox.value =
       serverType == "imap" || serverType == "nntp"
-        ? gMsgFolder.folderURL
-        : decodeURI(gMsgFolder.folderURL);
+        ? gMsgFolder.URI
+        : decodeURI(gMsgFolder.URI);
 
     if (gMsgFolder.canRename) {
       document.getElementById("name").removeAttribute("readonly");
@@ -237,42 +305,29 @@ function folderPropsOnLoad() {
 
     if (gMsgFolder.getFlag(Ci.nsMsgFolderFlags.Offline)) {
       if (serverType == "imap" || serverType == "pop3") {
-        document.getElementById(
-          "offline.selectForOfflineFolder"
-        ).checked = true;
+        document.getElementById("offline.selectForOfflineFolder").checked =
+          true;
       }
 
       if (serverType == "nntp") {
-        document.getElementById(
-          "offline.selectForOfflineNewsgroup"
-        ).checked = true;
+        document.getElementById("offline.selectForOfflineNewsgroup").checked =
+          true;
       }
     } else {
       if (serverType == "imap" || serverType == "pop3") {
-        document.getElementById(
-          "offline.selectForOfflineFolder"
-        ).checked = false;
+        document.getElementById("offline.selectForOfflineFolder").checked =
+          false;
       }
 
       if (serverType == "nntp") {
-        document.getElementById(
-          "offline.selectForOfflineNewsgroup"
-        ).checked = false;
+        document.getElementById("offline.selectForOfflineNewsgroup").checked =
+          false;
       }
     }
 
-    // select the menu item
-    var folderCharsetList = document.getElementById("folderCharsetList");
-    folderCharsetList.value = gMsgFolder.charset;
-
-    // set override checkbox
-    document.getElementById("folderCharsetOverride").checked =
-      gMsgFolder.charsetOverride;
-
     // set check for new mail checkbox
-    document.getElementById(
-      "folderCheckForNewMessages"
-    ).checked = gMsgFolder.getFlag(Ci.nsMsgFolderFlags.CheckNew);
+    document.getElementById("folderCheckForNewMessages").checked =
+      gMsgFolder.getFlag(Ci.nsMsgFolderFlags.CheckNew);
 
     // if gloda indexing is off, hide the related checkbox
     var glodaCheckbox = document.getElementById("folderIncludeInGlobalSearch");
@@ -287,17 +342,27 @@ function folderPropsOnLoad() {
       glodaCheckbox.hidden = true;
     } else {
       // otherwise, the user can choose whether this file gets indexed
-      let glodaFolder = Gloda.getFolderForFolder(gMsgFolder);
+      const glodaFolder = Gloda.getFolderForFolder(gMsgFolder);
       glodaCheckbox.checked =
         glodaFolder.indexingPriority != glodaFolder.kIndexingNeverPriority;
     }
   }
 
   if (serverType == "imap") {
-    var imapFolder = gMsgFolder.QueryInterface(Ci.nsIMsgImapMailFolder);
-    if (imapFolder) {
-      imapFolder.fillInFolderProps(gFolderPropsSink);
+    const imapFolder = gMsgFolder.QueryInterface(Ci.nsIMsgImapMailFolder);
+    imapFolder.fillInFolderProps(gFolderPropsSink);
+
+    const users = [...imapFolder.getOtherUsersWithAccess()];
+    if (users.length) {
+      document.getElementById("folderOtherUsers").hidden = false;
+      document.getElementById("folderOtherUsersText").textContent =
+        users.join(", ");
     }
+
+    // Disable "Repair Folder" when offline as that would cause the offline store
+    // to get deleted and redownloaded.
+    document.getElementById("folderRebuildSummaryButton").disabled =
+      gMsgFolder.supportsOffline && Services.io.offline;
   }
 
   var retentionSettings = gMsgFolder.retentionSettings;
@@ -306,68 +371,59 @@ function folderPropsOnLoad() {
     retentionSettings.useServerDefaults;
 
   // set folder sizes
-  let numberOfMsgs = gMsgFolder.getTotalMessages(false);
+  const numberOfMsgs = gMsgFolder.getTotalMessages(false);
   if (numberOfMsgs >= 0) {
     document.getElementById("numberOfMessages").value = numberOfMsgs;
   }
 
   try {
-    let sizeOnDisk = Cc["@mozilla.org/messenger;1"]
+    const sizeOnDisk = Cc["@mozilla.org/messenger;1"]
       .createInstance(Ci.nsIMessenger)
       .formatFileSize(gMsgFolder.sizeOnDisk, true);
     document.getElementById("sizeOnDisk").value = sizeOnDisk;
   } catch (e) {}
 
-  // select the initial tab
-  if (window.arguments[0].tabID) {
-    try {
-      document.getElementById(
-        "folderPropTabBox"
-      ).selectedTab = document.getElementById(window.arguments[0].tabID);
-    } catch (ex) {}
-  }
   onCheckKeepMsg();
   onUseDefaultRetentionSettings();
-}
 
-function hideShowControls(serverType) {
-  let controls = document.querySelectorAll("[hidefor]");
-  var len = controls.length;
-  for (var i = 0; i < len; i++) {
-    var control = controls[i];
-    var hideFor = control.getAttribute("hidefor");
-    if (!hideFor) {
-      throw new Error("hidefor empty");
-    }
-
-    // hide unsupported server type
-    // adding support for hiding multiple server types using hideFor="server1,server2"
-    var hideForBool = false;
-    var hideForTokens = hideFor.split(",");
-    for (var j = 0; j < hideForTokens.length; j++) {
-      if (hideForTokens[j] == serverType) {
-        hideForBool = true;
-        break;
-      }
-    }
-    control.hidden = hideForBool;
+  // select the initial tab
+  if (window.arguments[0].tabID) {
+    document.getElementById("folderPropTabBox").selectedTab =
+      document.getElementById(window.arguments[0].tabID);
   }
 
-  // hide the privileges button if the imap folder doesn't have an admin url
-  // mabye should leave this hidden by default and only show it in this case instead
-  try {
-    var imapFolder = gMsgFolder.QueryInterface(Ci.nsIMsgImapMailFolder);
-    if (imapFolder) {
-      var privilegesButton = document.getElementById("imap.FolderPrivileges");
-      if (privilegesButton) {
-        if (!imapFolder.hasAdminUrl) {
-          privilegesButton.setAttribute("hidden", "true");
-        }
-      }
-    }
-  } catch (ex) {}
+  // For click on the text about ".msf", output the folder Mork data to the
+  // native console for debugging purposes.
+  // If shift is pressed, use "raw" output, otherwise use prettified
+  // message header data only (there will be also folder meta data in the .msf).
+  document
+    .getElementById("folderRebuildSummaryExplanation")
+    .addEventListener("click", event => {
+      MorkParser.dumpFile(
+        gMsgFolder.summaryFile.path,
+        !event.shiftKey,
+        gMsgFolder.URI
+      );
+    });
+}
+
+/**
+ *  Hide or show controls based on server type.
+ *
+ * @param {string} serverType - The type of this server.
+ */
+function hideShowControls(serverType) {
+  for (const control of document.querySelectorAll("[hidefor]")) {
+    const hideForTypes = control.getAttribute("hidefor").split(",");
+    control.hidden = hideForTypes.includes(serverType);
+  }
 
   if (gMsgFolder) {
+    if (gMsgFolder instanceof Ci.nsIMsgImapMailFolder) {
+      document.getElementById("imap.FolderPrivileges").hidden =
+        !gMsgFolder.hasAdminUrl;
+    }
+
     // Hide "check for new mail" checkbox if this is an Inbox.
     if (gMsgFolder.getFlag(Ci.nsMsgFolderFlags.Inbox)) {
       document.getElementById("folderCheckForNewMessages").hidden = true;
@@ -391,12 +447,11 @@ function onOfflineFolderDownload() {
   gMsgFolder.downloadAllForOffline(null, window.arguments[0].msgWindow);
 }
 
+/** Open the folder privileges management url. */
 function onFolderPrivileges() {
-  var imapFolder = gMsgFolder.QueryInterface(Ci.nsIMsgImapMailFolder);
-  if (imapFolder) {
-    imapFolder.folderPrivileges(window.arguments[0].msgWindow);
-  }
-  // let's try closing the modal dialog to see if it fixes the various problems running this url
+  openLinkExternally(
+    gMsgFolder.QueryInterface(Ci.nsIMsgImapMailFolder).adminUrl
+  );
   window.close();
 }
 
@@ -407,13 +462,19 @@ function onUseDefaultRetentionSettings() {
   document.getElementById("retention.keepOldMsgMinLabel").disabled = useDefault;
 
   var keepMsg = document.getElementById("retention.keepMsg").value;
-  const nsIMsgRetentionSettings = Ci.nsIMsgRetentionSettings;
   document.getElementById("retention.keepOldMsgMin").disabled =
-    useDefault || keepMsg != nsIMsgRetentionSettings.nsMsgRetainByAge;
+    useDefault || keepMsg != Ci.nsIMsgRetentionSettings.nsMsgRetainByAge;
   document.getElementById("retention.keepNewMsgMin").disabled =
-    useDefault || keepMsg != nsIMsgRetentionSettings.nsMsgRetainByNumHeaders;
+    useDefault || keepMsg != Ci.nsIMsgRetentionSettings.nsMsgRetainByNumHeaders;
 }
 
-function RebuildSummaryInformation() {
-  window.arguments[0].rebuildSummaryCallback(gMsgFolder);
+/**
+ * Repair Folder.
+ *
+ * @param {Event} event - The click on the Repair Folder button.
+ */
+async function RebuildSummaryInformation(event) {
+  event.target.disabled = true;
+  await window.arguments[0].rebuildSummaryCallback(gMsgFolder);
+  event.target.disabled = false;
 }
