@@ -1,13 +1,18 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* import-globals-from dateFormat.js */
 
+var { UIFontSize } = ChromeUtils.importESModule(
+  "resource:///modules/UIFontSize.sys.mjs"
+);
+
 var MILLISECONDS_PER_HOUR = 60 * 60 * 1000;
 var MICROSECONDS_PER_DAY = 1000 * MILLISECONDS_PER_HOUR * 24;
 
+window.addEventListener("load", onLoad);
+
 document.addEventListener("dialogaccept", onAccept);
 
 function onLoad() {
@@ -24,6 +29,8 @@ function onLoad() {
   // we convert it to a date string, and then the time part is truncated
   upperDateBox.value = convertDateToString(initialDate);
   upperDateBox.select(); // allows to start overwriting immediately
+
+  UIFontSize.registerWindow(window);
 }
 
 function onAccept() {
@@ -59,13 +66,13 @@ function onAccept() {
   markInDatabase(prLower, prUpper);
 }
 
-/** marks all headers in the database, whose time is between the two
-  given times, as read.
-  @param lower
-    PRTime for the lower bound - this boundary is inclusive
-  @param upper
-    PRTime for the upper bound - this boundary is exclusive
-*/
+/**
+ * M arks all headers in the database, whose time is between the two given
+ * times, as read.
+ *
+ * @param {integer} lower - PRTime for the lower bound (inclusive).
+ * @param {integer} upper - PRTime for the upper bound (exclusive).
+ */
 function markInDatabase(lower, upper) {
   let messageFolder;
   let messageDatabase;
@@ -80,20 +87,17 @@ function markInDatabase(lower, upper) {
     return;
   }
 
-  let searchSession = Cc[
+  const searchSession = Cc[
     "@mozilla.org/messenger/searchSession;1"
   ].createInstance(Ci.nsIMsgSearchSession);
-  let searchTerms = [];
+  const searchTerms = [];
   searchSession.addScopeTerm(Ci.nsMsgSearchScope.offlineMail, messageFolder);
 
-  const nsMsgSearchAttrib = Ci.nsMsgSearchAttrib;
-  const nsMsgSearchOp = Ci.nsMsgSearchOp;
-
   let searchTerm = searchSession.createTerm();
-  searchTerm.attrib = nsMsgSearchAttrib.Date;
-  searchTerm.op = nsMsgSearchOp.IsBefore;
+  searchTerm.attrib = Ci.nsMsgSearchAttrib.Date;
+  searchTerm.op = Ci.nsMsgSearchOp.IsBefore;
   let value = searchTerm.value;
-  value.attrib = nsMsgSearchAttrib.Date;
+  value.attrib = Ci.nsMsgSearchAttrib.Date;
   value.date = upper;
   searchTerm.value = value;
   searchTerms.push(searchTerm);
@@ -101,17 +105,17 @@ function markInDatabase(lower, upper) {
   if (lower) {
     searchTerm = searchSession.createTerm();
     searchTerm.booleanAnd = true;
-    searchTerm.attrib = nsMsgSearchAttrib.Date;
-    searchTerm.op = nsMsgSearchOp.IsAfter;
+    searchTerm.attrib = Ci.nsMsgSearchAttrib.Date;
+    searchTerm.op = Ci.nsMsgSearchOp.IsAfter;
     value = searchTerm.value;
-    value.attrib = nsMsgSearchAttrib.Date;
+    value.attrib = Ci.nsMsgSearchAttrib.Date;
     value.date = lower;
     searchTerm.value = value;
     searchTerms.push(searchTerm);
   }
 
-  let msgEnumerator = messageDatabase.getFilterEnumerator(searchTerms);
-  let headers = [...msgEnumerator];
+  const msgEnumerator = messageDatabase.getFilterEnumerator(searchTerms);
+  const headers = [...msgEnumerator];
 
   if (headers.length) {
     messageFolder.markMessagesRead(headers, true);
