@@ -26,12 +26,8 @@
 #include "nsTArray.h"
 
 /* DBFCFDF0-4489-4faa-8122-190FD1EFA16C */
-#define NS_MSGCONTENTPOLICY_CID                     \
-  {                                                 \
-    0xdbfcfdf0, 0x4489, 0x4faa, {                   \
-      0x81, 0x22, 0x19, 0xf, 0xd1, 0xef, 0xa1, 0x6c \
-    }                                               \
-  }
+#define NS_MSGCONTENTPOLICY_CID \
+  {0xdbfcfdf0, 0x4489, 0x4faa, {0x81, 0x22, 0x19, 0xf, 0xd1, 0xef, 0xa1, 0x6c}}
 
 #define NS_MSGCONTENTPOLICY_CONTRACTID "@mozilla.org/messenger/content-policy;1"
 
@@ -69,16 +65,18 @@ class nsMsgContentPolicy : public nsIContentPolicy,
   int16_t ShouldAcceptRemoteContentForMsgHdr(nsIMsgDBHdr* aMsgHdr,
                                              nsIURI* aRequestingLocation,
                                              nsIURI* aContentLocation);
-  void NotifyContentWasBlocked(nsIURI* aOriginatorLocation,
-                               nsIURI* aContentLocation, bool aCanOverride);
-  void ShouldAcceptContentForPotentialMsg(nsIURI* aOriginatorLocation,
+  void NotifyContentWasBlocked(uint64_t aBrowsingContextId,
+                               nsIURI* aContentLocation);
+  void ShouldAcceptContentForPotentialMsg(uint64_t aBrowsingContextId,
+                                          nsIURI* aOriginatorLocation,
                                           nsIURI* aContentLocation,
                                           int16_t* aDecision);
   void ComposeShouldLoad(nsIMsgCompose* aMsgCompose,
                          nsISupports* aRequestingContext,
-                         nsIURI* aContentLocation, int16_t* aDecision);
-  already_AddRefed<nsIMsgCompose> GetMsgComposeForContext(
-      nsISupports* aRequestingContext);
+                         nsIURI* aOriginatorLocation, nsIURI* aContentLocation,
+                         int16_t* aDecision);
+  already_AddRefed<nsIMsgCompose> GetMsgComposeForBrowsingContext(
+      mozilla::dom::BrowsingContext* aRequestingContext);
 
   nsresult GetRootDocShellForContext(nsISupports* aRequestingContext,
                                      nsIDocShell** aDocShell);
