@@ -6,22 +6,18 @@
 #ifndef nsMsgMailNewsUrl_h___
 #define nsMsgMailNewsUrl_h___
 
+#include "msgCore.h"
 #include "nscore.h"
 #include "nsISupports.h"
 #include "nsIUrlListener.h"
 #include "nsTObserverArray.h"
-#include "nsIMsgWindow.h"
-#include "nsIMsgStatusFeedback.h"
 #include "nsCOMPtr.h"
-#include "nsCOMArray.h"
 #include "nsIMimeHeaders.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIURL.h"
 #include "nsIURIWithSpecialOrigin.h"
-#include "nsILoadGroup.h"
 #include "nsIMsgSearchSession.h"
 #include "nsICacheEntry.h"
-#include "nsIMimeMiscStatus.h"
 #include "nsIWeakReferenceUtils.h"
 #include "nsString.h"
 #include "nsIURIMutator.h"
@@ -37,10 +33,10 @@
 //
 //////////////////////////////////////////////////////////////////////////////////
 
-class NS_MSG_BASE nsMsgMailNewsUrl : public nsIMsgMailNewsUrl,
-                                     public nsIURIWithSpecialOrigin,
-                                     public nsISerializable,
-                                     public nsIClassInfo {
+class nsMsgMailNewsUrl : public nsIMsgMailNewsUrl,
+                         public nsIURIWithSpecialOrigin,
+                         public nsISerializable,
+                         public nsIClassInfo {
  public:
   nsMsgMailNewsUrl();
 
@@ -111,8 +107,8 @@ class NS_MSG_BASE nsMsgMailNewsUrl : public nsIMsgMailNewsUrl,
   nsCOMPtr<nsIMimeHeaders> mMimeHeaders;
   nsCOMPtr<nsIMsgSearchSession> m_searchSession;
   nsCOMPtr<nsICacheEntry> m_memCacheEntry;
-  nsCOMPtr<nsIMsgHeaderSink> mMsgHeaderSink;
   nsCString m_errorCode;
+  nsCString m_seeOtherURI;
   nsString m_errorMessage;
   nsString m_errorParameters;
   int64_t mMaxProgress;
