@@ -9,14 +9,11 @@
 #include "nsIMsgMailSession.h"
 #include "nsISupports.h"
 #include "nsCOMPtr.h"
-#include "nsIMsgStatusFeedback.h"
 #include "nsIMsgWindow.h"
 #include "nsCOMArray.h"
 #include "nsIMsgShutdown.h"
 #include "nsIObserver.h"
-#include "nsIMutableArray.h"
 #include "nsIMsgProgress.h"
-#include "nsTArray.h"
 #include "nsTObserverArray.h"
 #include "nsIMsgUserFeedbackListener.h"
 #include "nsIUrlListener.h"
