@@ -14,14 +14,19 @@
 #include "nsIComponentRegistrar.h"
 #include "nsXULAppAPI.h"
 
-#include "../../local/src/nsPop3Service.h"
+#include "../../local/src/nsPop3URL.h"
 #include "../../local/src/nsMailboxService.h"
-#include "../../compose/src/nsSmtpService.h"
-#include "../../../ldap/xpcom/src/nsLDAPURL.h"
+#include "../../compose/src/nsSmtpUrl.h"
+#include "../../addrbook/src/nsLDAPURL.h"
 #include "../../imap/src/nsImapService.h"
-#include "../../news/src/nsNntpService.h"
+#include "../../news/src/nsNntpUrl.h"
 #include "../src/nsCidProtocolHandler.h"
 
+// Instantiates a new `nsIURI` of the appropriate concrete type for the provided
+// URI spec.
+//
+// This is primarily intended to be called by `NS_NewURI` in necko for
+// handling Thunderbird-specific URI schemes.
 nsresult NS_NewMailnewsURI(nsIURI** aURI, const nsACString& aSpec,
                            const char* aCharset /* = nullptr */,
                            nsIURI* aBaseURI /* = nullptr */) {
@@ -42,47 +47,56 @@ nsresult NS_NewMailnewsURI(nsIURI** aURI, const nsACString& aSpec,
     if (NS_FAILED(rv)) return rv;
   }
 
+  // Creating IMAP/mailbox URIs off the main thread can lead to crashes.
+  // Seems to happen when viewing PDFs.
   if (scheme.EqualsLiteral("mailbox") ||
       scheme.EqualsLiteral("mailbox-message")) {
-    return nsMailboxService::NewURI(aSpec, aCharset, aBaseURI, aURI);
+    if (NS_IsMainThread()) {
+      return nsMailboxService::NewURI(aSpec, aCharset, aBaseURI, aURI);
+    }
+    auto NewURI = [&aSpec, &aCharset, &aBaseURI, aURI, &rv]() -> auto {
+      rv = nsMailboxService::NewURI(aSpec, aCharset, aBaseURI, aURI);
+    };
+    nsCOMPtr<nsIRunnable> task = NS_NewRunnableFunction("NewURI", NewURI);
+    mozilla::SyncRunnable::DispatchToThread(
+        mozilla::GetMainThreadSerialEventTarget(), task);
+    return rv;
   }
   if (scheme.EqualsLiteral("imap") || scheme.EqualsLiteral("imap-message")) {
     if (NS_IsMainThread()) {
       return nsImapService::NewURI(aSpec, aCharset, aBaseURI, aURI);
     }
-    // Creating IMAP URIs off the main tread can lead to crashes.
-    // Seems to happen when viewing PDFs.
-    auto NewURI = [&aSpec, &aCharset, &aBaseURI, aURI, &rv ]() -> auto {
+    auto NewURI = [&aSpec, &aCharset, &aBaseURI, aURI, &rv]() -> auto {
       rv = nsImapService::NewURI(aSpec, aCharset, aBaseURI, aURI);
     };
     nsCOMPtr<nsIRunnable> task = NS_NewRunnableFunction("NewURI", NewURI);
-    mozilla::SyncRunnable::DispatchToThread(mozilla::GetMainThreadEventTarget(),
-                                            task);
+    mozilla::SyncRunnable::DispatchToThread(
+        mozilla::GetMainThreadSerialEventTarget(), task);
     return rv;
   }
   if (scheme.EqualsLiteral("smtp") || scheme.EqualsLiteral("smtps")) {
-    return nsSmtpService::NewSmtpURI(aSpec, aCharset, aBaseURI, aURI);
+    return nsSmtpUrl::NewSmtpURI(aSpec, aBaseURI, aURI);
   }
   if (scheme.EqualsLiteral("mailto")) {
     if (NS_IsMainThread()) {
-      return nsSmtpService::NewMailtoURI(aSpec, aCharset, aBaseURI, aURI);
+      return nsMailtoUrl::NewMailtoURI(aSpec, aBaseURI, aURI);
     }
     // If we're for some reason not on the main thread, dispatch to main
     // or else we'll crash.
-    auto NewURI = [&aSpec, &aCharset, &aBaseURI, aURI, &rv ]() -> auto {
-      rv = nsSmtpService::NewMailtoURI(aSpec, aCharset, aBaseURI, aURI);
+    auto NewURI = [&aSpec, &aBaseURI, aURI, &rv]() -> auto {
+      rv = nsMailtoUrl::NewMailtoURI(aSpec, aBaseURI, aURI);
     };
     nsCOMPtr<nsIRunnable> task = NS_NewRunnableFunction("NewURI", NewURI);
-    mozilla::SyncRunnable::DispatchToThread(mozilla::GetMainThreadEventTarget(),
-                                            task);
+    mozilla::SyncRunnable::DispatchToThread(
+        mozilla::GetMainThreadSerialEventTarget(), task);
     return rv;
   }
   if (scheme.EqualsLiteral("pop") || scheme.EqualsLiteral("pop3")) {
-    return nsPop3Service::NewURI(aSpec, aCharset, aBaseURI, aURI);
+    return nsPop3URL::NewURI(aSpec, aBaseURI, aURI);
   }
   if (scheme.EqualsLiteral("news") || scheme.EqualsLiteral("snews") ||
       scheme.EqualsLiteral("news-message") || scheme.EqualsLiteral("nntp")) {
-    return nsNntpService::NewURI(aSpec, aCharset, aBaseURI, aURI);
+    return nsNntpUrl::NewURI(aSpec, aBaseURI, aURI);
   }
   if (scheme.EqualsLiteral("cid")) {
     return nsCidProtocolHandler::NewURI(aSpec, aCharset, aBaseURI, aURI);
@@ -112,6 +126,13 @@ nsresult NS_NewMailnewsURI(nsIURI** aURI, const nsACString& aSpec,
         .SetSpec(aSpec)
         .Finalize(aURI);
   }
+#if defined(MOZ_THUNDERBIRD_RUST)
+  if (scheme.EqualsLiteral("ews") || scheme.EqualsLiteral("ews-message")) {
+    return NS_MutateURI(new mozilla::net::nsStandardURL::Mutator())
+        .SetSpec(aSpec)
+        .Finalize(aURI);
+  }
+#endif
 
   rv = NS_ERROR_UNKNOWN_PROTOCOL;  // Let M-C handle it by default.
 
@@ -123,8 +144,8 @@ nsresult NS_NewMailnewsURI(nsIURI** aURI, const nsACString& aSpec,
     bool isRegistered = false;
     compMgr->IsContractIDRegistered(contractID.get(), &isRegistered);
     if (isRegistered) {
-      auto NewURI =
-          [&aSpec, &aCharset, &aBaseURI, aURI, &contractID, &rv ]() -> auto {
+      auto NewURI = [&aSpec, &aCharset, &aBaseURI, aURI, &contractID,
+                     &rv]() -> auto {
         nsCOMPtr<nsIMsgProtocolHandler> handler(
             do_GetService(contractID.get()));
         if (handler) {
@@ -137,7 +158,7 @@ nsresult NS_NewMailnewsURI(nsIURI** aURI, const nsACString& aSpec,
       } else {
         nsCOMPtr<nsIRunnable> task = NS_NewRunnableFunction("NewURI", NewURI);
         mozilla::SyncRunnable::DispatchToThread(
-            mozilla::GetMainThreadEventTarget(), task);
+            mozilla::GetMainThreadSerialEventTarget(), task);
       }
     }
   }
