@@ -2,13 +2,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/*
+/**
  * This file tests our querying support.  We build up a deterministic little
  *  'world' of messages spread across multiple conversations, multiple folders
  *  and multiple authors.  To verify expected negative results, in addition to
  *  the 'peoples' in our world clique, we also have 'outlier' contacts that do
  *  not communicate with the others (but are also spread across folders).
  *
+ * Furthermore, we test that messages with correspondents lacking valid email
+ * addresses can also be successfully queried.
+ *
  * This is broadly intended to test all of our query features and mechanisms
  *  (apart from our specialized search implementation, which is tested by
  *  test_search_messages.js), but is probably not the place to test specific
@@ -19,8 +22,25 @@
  *  define a bunch of message corpuses entirely specialized for each test.
  */
 
-/* import-globals-from resources/glodaTestHelper.js */
-load("resources/glodaTestHelper.js");
+var { assertExpectedMessagesIndexed, waitForGlodaIndexer } =
+  ChromeUtils.importESModule(
+    "resource://testing-common/gloda/GlodaTestHelper.sys.mjs"
+  );
+var { queryExpect } = ChromeUtils.importESModule(
+  "resource://testing-common/gloda/GlodaQueryHelper.sys.mjs"
+);
+var { Gloda } = ChromeUtils.importESModule(
+  "resource:///modules/gloda/GlodaPublic.sys.mjs"
+);
+var { GlodaConstants } = ChromeUtils.importESModule(
+  "resource:///modules/gloda/GlodaConstants.sys.mjs"
+);
+var { SyntheticMessageSet } = ChromeUtils.importESModule(
+  "resource://testing-common/mailnews/MessageGenerator.sys.mjs"
+);
+
+var msgGen;
+var messageInjection;
 
 /**
  * Whether we expect fulltext results. IMAP folders that are offline shouldn't
@@ -38,41 +58,46 @@ var goOffline = false;
 var world = {
   phase: 0,
 
-  // a list of tuples of [name, email] of length NUM_AUTHORS
+  // A list of tuples of [name, email] of length NUM_AUTHORS.
   peoples: null,
   NUM_AUTHORS: 5,
-  // maps each author (as defined by their email address) to the list of
-  //  (synthetic) messages they have 'authored'
+  // Maps each author (as defined by their email address) to the list of
+  //  (synthetic) messages they have 'authored'.
   authorGroups: {},
 
   NUM_CONVERSATIONS: 3,
-  // the last message (so far) in each conversation
+  // The last message (so far) in each conversation.
   lastMessagesInConvos: [],
-  // maps the message-id of the root message in a conversation to the list of
-  //  synthetic messages in the conversation
+  // Maps the message-id of the root message in a conversation to the list of
+  //  synthetic messages in the conversation.
   conversationGroups: {},
-  // a list of lists of synthetic messages, organized by the conversation they
+  // A list of lists of synthetic messages, organized by the conversation they
   //  belong to.
   conversationLists: [],
-  // a list of gloda conversation id's, each corresponding to the entries in
-  // converastionLists.
+  // A list of gloda conversation id's, each corresponding to the entries in
+  //  converastionLists.
   glodaConversationIds: [],
 
   NUM_FOLDERS: 2,
   MESSAGES_PER_FOLDER: 11,
-  // a list of lists of synthetic messages, one list per folder
+  // A list of lists of synthetic messages, one list per folder.
   folderClumps: [],
-  // a list of nsIMsgFolders, with each folder containing the messages in the
-  //  corresponding list in folderClumps
+  // A list of nsIMsgFolders, with each folder containing the messages in the
+  //  corresponding list in folderClumps.
   glodaFolders: [],
 
   outlierAuthor: null,
   outlierFriend: null,
 
-  // messages authored by contacts in the "peoples" group
+  maillessAuthor: null,
+  maillessFriend: null,
+
+  // Messages authored by contacts in the "peoples" group.
   peoplesMessages: [],
-  // messages authored by outlierAuthor and outlierFriend
+  // Messages authored by outlierAuthor and outlierFriend.
   outlierMessages: [],
+  // Messages authored by maillessAuthor and maillessFriend.
+  maillessMessages: [],
 };
 
 /**
@@ -85,7 +110,7 @@ var world = {
 function uniqueTermGenerator(aNum) {
   let s = "uniq";
   do {
-    let l = String.fromCharCode(97 + (aNum % 26));
+    const l = String.fromCharCode(97 + (aNum % 26));
     s += l + l;
     aNum = Math.floor(aNum / 26);
   } while (aNum);
@@ -103,17 +128,17 @@ var UNIQUE_OFFSET_ATTACHMENT = 26 * 26 * 26;
  *  structure.  This is then used by the test code to generate and verify query
  *  data.
  *
- * @param aSynthMessage The synthetic message.
+ * @param {SyntheticMessage} aSynthMessage The synthetic message.
  */
 function categorizeMessage(aSynthMessage) {
-  // lump by author
-  let author = aSynthMessage.fromAddress;
+  // Lump by author.
+  const author = aSynthMessage.fromAddress;
   if (!(author in world.authorGroups)) {
     world.authorGroups[author] = [];
   }
   world.authorGroups[author].push(aSynthMessage);
 
-  // lump by conversation, keying off of the originator's message id
+  // Lump by conversation, keying off of the originator's message id.
   let originator = aSynthMessage;
   while (originator.parent) {
     originator = originator.parent;
@@ -124,7 +149,7 @@ function categorizeMessage(aSynthMessage) {
   world.conversationGroups[originator.messageId].push(aSynthMessage);
   world.conversationLists[aSynthMessage.iConvo].push(aSynthMessage);
 
-  // folder lumping happens in a big glob
+  // Folder lumping happens in a big glob.
 }
 
 /**
@@ -135,35 +160,35 @@ function categorizeMessage(aSynthMessage) {
  *   including themselves.
  */
 function generateFolderMessages() {
-  let messages = [],
-    smsg;
+  const messages = [];
+  let smsg;
 
   let iAuthor = 0;
   for (let iMessage = 0; iMessage < world.MESSAGES_PER_FOLDER; iMessage++) {
-    let iConvo = iMessage % world.NUM_CONVERSATIONS;
+    const iConvo = iMessage % world.NUM_CONVERSATIONS;
 
-    // we need missing messages to create ghosts, so periodically add an extra
+    // We need missing messages to create ghosts, so periodically add an extra
     //  unknown into the equation.  we do this prior to the below step because
-    //  then we don't hose up all the fancy body creation the next step does
+    //  then we don't hose up all the fancy body creation the next step does.
     if (iMessage % 3 == 1) {
       smsg = msgGen.makeMessage({ inReplyTo: smsg });
     }
 
-    let convUniqueSubject = uniqueTermGenerator(
+    const convUniqueSubject = uniqueTermGenerator(
       UNIQUE_OFFSET_SUBJECT + UNIQUE_OFFSET_CONV + iConvo
     );
-    let convUniqueBody = uniqueTermGenerator(
+    const convUniqueBody = uniqueTermGenerator(
       UNIQUE_OFFSET_BODY + UNIQUE_OFFSET_CONV + iConvo
     );
-    let authorUniqueBody = uniqueTermGenerator(
+    const authorUniqueBody = uniqueTermGenerator(
       UNIQUE_OFFSET_BODY + UNIQUE_OFFSET_AUTHOR + iAuthor
     );
-    let convUniqueAttachment = uniqueTermGenerator(
+    const convUniqueAttachment = uniqueTermGenerator(
       UNIQUE_OFFSET_ATTACHMENT + UNIQUE_OFFSET_CONV + iConvo
     );
     smsg = msgGen.makeMessage({
       inReplyTo: world.lastMessagesInConvos[iConvo],
-      // note that the reply-logic will ignore our subject, luckily that does
+      // Note that the reply-logic will ignore our subject, luckily that does
       //  not matter! (since it will just copy the subject)
       subject: convUniqueSubject,
       body: {
@@ -178,14 +203,14 @@ function generateFolderMessages() {
       ],
     });
 
-    // makeMessage is not exceedingly clever right now, we need to overwrite
-    //  From and To...
+    // MakeMessage is not exceedingly clever right now, we need to overwrite
+    //  From and To.
     smsg.from = world.peoples[iAuthor];
     iAuthor = (iAuthor + iConvo + 1) % world.NUM_AUTHORS;
-    // so, everyone is talking to everyone for this stuff
+    // So, everyone is talking to everyone for this stuff.
     smsg.to = world.peoples;
     world.lastMessagesInConvos[iConvo] = smsg;
-    // simplify categorizeMessage and glodaInfoStasher's life
+    // Simplify categorizeMessage and glodaInfoStasher's life.
     smsg.iConvo = iConvo;
 
     categorizeMessage(smsg);
@@ -196,10 +221,20 @@ function generateFolderMessages() {
   smsg = msgGen.makeMessage();
   smsg.from = world.outlierAuthor;
   smsg.to = [world.outlierFriend];
-  // do not lump it
+  // Do not lump it.
   messages.push(smsg);
   world.outlierMessages.push(smsg);
 
+  smsg = msgGen.makeMessage({
+    clobberHeaders: {
+      From: world.maillessAuthor,
+      To: world.maillessFriend,
+    },
+  });
+  // Do not lump it.
+  messages.push(smsg);
+  world.maillessMessages.push(smsg);
+
   world.folderClumps.push(messages);
 
   return new SyntheticMessageSet(messages);
@@ -220,49 +255,55 @@ function glodaInfoStasher(aSynthMessage, aGlodaMessage) {
   }
 }
 
-// We override these for the IMAP tests
-var pre_setup_populate_hook = function default_pre_setup_populate_hook() {};
-var post_setup_populate_hook = function default_post_setup_populate_hook() {};
-
-// first, we must populate our message store with delicious messages.
-function* setup_populate() {
+// First, we must populate our message store with delicious messages.
+async function setup_populate() {
   world.glodaHolderCollection = Gloda.explicitCollection(
-    Gloda.NOUN_MESSAGE,
+    GlodaConstants.NOUN_MESSAGE,
     []
   );
 
   world.peoples = msgGen.makeNamesAndAddresses(world.NUM_AUTHORS);
   world.outlierAuthor = msgGen.makeNameAndAddress();
   world.outlierFriend = msgGen.makeNameAndAddress();
-  // set up the per-conversation values with blanks initially
+  world.maillessAuthor = "mailless.author at example.com (Mailless Author)";
+  world.maillessFriend = "Mailless Friend";
+  // Set up the per-conversation values with blanks initially.
   for (let iConvo = 0; iConvo < world.NUM_CONVERSATIONS; iConvo++) {
     world.lastMessagesInConvos.push(null);
     world.conversationLists.push([]);
     world.glodaConversationIds.push(null);
   }
 
-  let setOne = generateFolderMessages();
-  let folderOne = make_empty_folder();
-  yield add_sets_to_folders(folderOne, [setOne]);
+  const setOne = generateFolderMessages();
+  const folderOne = await messageInjection.makeEmptyFolder();
+  await messageInjection.addSetsToFolders([folderOne], [setOne]);
   // If this is the online_to_offline variant (indicated by goOffline) we want
   //  to make the messages available offline.  This should trigger an event
   //  driven re-indexing of the messages which should make the body available
   //  for fulltext queries.
   if (goOffline) {
-    yield wait_for_gloda_indexer(setOne);
-    yield make_folder_and_contents_offline(folderOne);
+    await waitForGlodaIndexer();
+    Assert.ok(...assertExpectedMessagesIndexed([setOne]));
+    await messageInjection.makeFolderAndContentsOffline(folderOne);
   }
-  yield wait_for_gloda_indexer(setOne, { verifier: glodaInfoStasher });
+  await waitForGlodaIndexer();
+  Assert.ok(
+    ...assertExpectedMessagesIndexed([setOne], { verifier: glodaInfoStasher })
+  );
 
   world.phase++;
-  let setTwo = generateFolderMessages();
-  let folderTwo = make_empty_folder();
-  yield add_sets_to_folders(folderTwo, [setTwo]);
+  const setTwo = generateFolderMessages();
+  const folderTwo = await messageInjection.makeEmptyFolder();
+  await messageInjection.addSetsToFolders([folderTwo], [setTwo]);
   if (goOffline) {
-    yield wait_for_gloda_indexer(setTwo);
-    yield make_folder_and_contents_offline(folderTwo);
+    await waitForGlodaIndexer();
+    Assert.ok(...assertExpectedMessagesIndexed([setTwo]));
+    await messageInjection.makeFolderAndContentsOffline(folderTwo);
   }
-  yield wait_for_gloda_indexer(setTwo, { verifier: glodaInfoStasher });
+  await waitForGlodaIndexer();
+  Assert.ok(
+    ...assertExpectedMessagesIndexed([setTwo], { verifier: glodaInfoStasher })
+  );
 }
 
 /* ===== Non-text queries ===== */
@@ -278,10 +319,10 @@ function* setup_populate() {
  */
 function verify_nonMatches(aQueries, aCollections) {
   for (let i = 0; i < aCollections.length; i++) {
-    let testQuery = aQueries[i];
-    let nonmatches = aCollections[(i + 1) % aCollections.length].items;
+    const testQuery = aQueries[i];
+    const nonmatches = aCollections[(i + 1) % aCollections.length].items;
 
-    for (let item of nonmatches) {
+    for (const item of nonmatches) {
       if (testQuery.test(item)) {
         dump("item: " + JSON.stringify(item) + "\n");
         dump("constraints: " + JSON.stringify(testQuery._constraints) + "\n");
@@ -305,21 +346,22 @@ var ts_convCollections = [];
  *  use of test_query_messages_by_conversation_nonmatches who verifies the
  *  query.test() logic doesn't match on things it should not match on.
  *
- * @tests gloda.noun.message.attr.conversation
- * @tests gloda.datastore.sqlgen.kConstraintIn
+ * Tests gloda.noun.message.attr.conversation
+ * Tests gloda.datastore.sqlgen.kConstraintIn
  */
-function test_query_messages_by_conversation() {
-  let convNum = ts_convNum++;
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_conversation() {
+  const convNum = ts_convNum++;
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.conversation(world.glodaConversationIds[convNum]);
 
   ts_convQueries.push(query);
-  ts_convCollections.push(queryExpect(query, world.conversationLists[convNum]));
-  return false; // async pend on queryExpect
+  ts_convCollections.push(
+    await queryExpect(query, world.conversationLists[convNum])
+  );
 }
 
 /**
- * @tests gloda.query.test.kConstraintIn
+ * Tests gloda.query.test.kConstraintIn
  */
 function test_query_messages_by_conversation_nonmatches() {
   verify_nonMatches(ts_convQueries, ts_convCollections);
@@ -329,99 +371,115 @@ var ts_folderNum = 0;
 var ts_folderQueries = [];
 var ts_folderCollections = [];
 /**
- * @tests gloda.noun.message.attr.folder
- * @tests gloda.datastore.sqlgen.kConstraintIn
+ * Tests gloda.noun.message.attr.folder
+ * Tests gloda.datastore.sqlgen.kConstraintIn
  */
-function test_query_messages_by_folder() {
-  let folderNum = ts_folderNum++;
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_folder() {
+  const folderNum = ts_folderNum++;
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.folder(world.glodaFolders[folderNum]);
 
   ts_folderQueries.push(query);
-  ts_folderCollections.push(queryExpect(query, world.folderClumps[folderNum]));
-  return false; // async pend on queryExpect
+  ts_folderCollections.push(
+    await queryExpect(query, world.folderClumps[folderNum])
+  );
 }
 
 /**
- * @tests gloda.query.test.kConstraintIn
+ * Tests gloda.query.test.kConstraintIn
  */
 function test_query_messages_by_folder_nonmatches() {
   verify_nonMatches(ts_folderQueries, ts_folderCollections);
 }
 
 /**
- * @tests Gloda.ns.getMessageCollectionForHeader()
+ * Tests Gloda.ns.getMessageCollectionForHeader()
  */
-function test_get_message_for_header() {
-  // pick an arbitrary message
-  let glodaMessage = ts_convCollections[1].items[0];
-  // find the synthetic message that matches (ordering must not be assumed)
-  let synthMessage = world.conversationLists[1].find(
+async function test_get_message_for_header() {
+  // Pick an arbitrary message.
+  const glodaMessage = ts_convCollections[1].items[0];
+  // Find the synthetic message that matches (ordering must not be assumed).
+  const synthMessage = world.conversationLists[1].find(
     sm => sm.messageId == glodaMessage.headerMessageID
   );
-  queryExpect(
+  await queryExpect(
     {
       queryFunc: Gloda.getMessageCollectionForHeader,
       queryThis: Gloda,
       args: [glodaMessage.folderMessage],
-      nounId: Gloda.NOUN_MESSAGE,
+      nounId: GlodaConstants.NOUN_MESSAGE,
     },
     [synthMessage]
   );
-  return false; // async pend on queryExpect
 }
 
 /**
- * @tests Gloda.ns.getMessageCollectionForHeaders()
+ * Tests Gloda.ns.getMessageCollectionForHeaders()
  */
-function test_get_messages_for_headers() {
-  let messageCollection = ts_convCollections[0];
-  let headers = messageCollection.items.map(m => m.folderMessage);
-  queryExpect(
+async function test_get_messages_for_headers() {
+  const messageCollection = ts_convCollections[0];
+  const headers = messageCollection.items.map(m => m.folderMessage);
+  await queryExpect(
     {
       queryFunc: Gloda.getMessageCollectionForHeaders,
       queryThis: Gloda,
       args: [headers],
-      nounId: Gloda.NOUN_MESSAGE,
+      nounId: GlodaConstants.NOUN_MESSAGE,
     },
     world.conversationLists[0]
   );
-  return false; // async pend on queryExpect
 }
 
-// at this point we go run the identity and contact tests for side-effects
+// At this point we go run the identity and contact tests for side-effects.
 
 var ts_messageIdentityQueries = [];
 var ts_messageIdentityCollections = [];
 /**
- * @tests gloda.noun.message.attr.involves
- * @tests gloda.datastore.sqlgen.kConstraintIn
+ * Tests gloda.noun.message.attr.involves
+ * Tests gloda.datastore.sqlgen.kConstraintIn
  */
-function test_query_messages_by_identity_peoples() {
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_identity_peoples() {
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.involves(peoplesIdentityCollection.items[0]);
 
   ts_messageIdentityQueries.push(query);
-  ts_messageIdentityCollections.push(queryExpect(query, world.peoplesMessages));
-  return false; // async pend on queryExpect
+  ts_messageIdentityCollections.push(
+    await queryExpect(query, world.peoplesMessages)
+  );
 }
 
 /**
- * @tests gloda.noun.message.attr.involves
+ * Tests gloda.noun.message.attr.involves
  */
-function test_query_messages_by_identity_outlier() {
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_identity_outlier() {
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.involves(outlierIdentityCollection.items[0]);
-  // this also tests our ability to have two intersecting constraints! hooray!
+  // This also tests our ability to have two intersecting constraints! hooray!.
   query.involves(outlierIdentityCollection.items[1]);
 
   ts_messageIdentityQueries.push(query);
-  ts_messageIdentityCollections.push(queryExpect(query, world.outlierMessages));
-  return false; // async pend on queryExpect
+  ts_messageIdentityCollections.push(
+    await queryExpect(query, world.outlierMessages)
+  );
 }
 
 /**
- * @tests gloda.query.test.kConstraintIn
+ * Tests gloda.noun.message.attr.involves
+ */
+async function test_query_messages_by_identity_mailless() {
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
+  query.involves(maillessIdentityCollection.items[0]);
+  // This also tests our ability to have two intersecting constraints! hooray!.
+  query.involves(maillessIdentityCollection.items[1]);
+
+  ts_messageIdentityQueries.push(query);
+  ts_messageIdentityCollections.push(
+    await queryExpect(query, world.maillessMessages)
+  );
+}
+
+/**
+ * Tests gloda.query.test.kConstraintIn
  */
 function test_query_messages_by_identity_nonmatches() {
   verify_nonMatches(ts_messageIdentityQueries, ts_messageIdentityCollections);
@@ -434,23 +492,22 @@ function test_query_messages_by_contact() {
 
 var ts_messagesDateQuery;
 /**
- * @tests gloda.noun.message.attr.date
- * @tests gloda.datastore.sqlgen.kConstraintRanges
+ * Tests gloda.noun.message.attr.date
+ * Tests gloda.datastore.sqlgen.kConstraintRanges
  */
-function test_query_messages_by_date() {
-  ts_messagesDateQuery = Gloda.newQuery(Gloda.NOUN_MESSAGE);
-  // we are clearly relying on knowing the generation sequence here,
-  //  fuggedaboutit
+async function test_query_messages_by_date() {
+  ts_messagesDateQuery = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
+  // We are clearly relying on knowing the generation sequence here,
+  //  fuggedaboutit.
   ts_messagesDateQuery.dateRange([
     world.peoplesMessages[1].date,
     world.peoplesMessages[2].date,
   ]);
-  queryExpect(ts_messagesDateQuery, world.peoplesMessages.slice(1, 3));
-  return false; // async pend on queryExpect
+  await queryExpect(ts_messagesDateQuery, world.peoplesMessages.slice(1, 3));
 }
 
 /**
- * @tests gloda.query.test.kConstraintRanges
+ * Tests gloda.query.test.kConstraintRanges
  */
 function test_query_messages_by_date_nonmatches() {
   if (
@@ -481,89 +538,84 @@ function test_query_conversations_by_subject_text() {}
 /**
  * Test subject searching using the conversation unique subject term.
  *
- * @tests gloda.noun.message.attr.subjectMatches
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.subjectMatches
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_subject_text() {
-  // we only need to use one conversation
-  let convNum = 0;
-  // dump("convNum: " + convNum + " blah: " + world.conversationLists[convNum] + "\n");
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
-  let convSubjectTerm = uniqueTermGenerator(
+async function test_query_messages_by_subject_text() {
+  // We only need to use one conversation.
+  const convNum = 0;
+
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
+  const convSubjectTerm = uniqueTermGenerator(
     UNIQUE_OFFSET_SUBJECT + UNIQUE_OFFSET_CONV + convNum
   );
   query.subjectMatches(convSubjectTerm);
-  queryExpect(query, world.conversationLists[convNum]);
-  return false; // async pend on queryExpect
+  await queryExpect(query, world.conversationLists[convNum]);
 }
 
 /**
  * Test body searching using the conversation unique body term.
  *
- * @tests gloda.noun.message.attr.bodyMatches
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.bodyMatches
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_body_text() {
-  // we only need to use one conversation
-  let convNum = 0;
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
-  let convBodyTerm = uniqueTermGenerator(
+async function test_query_messages_by_body_text() {
+  // We only need to use one conversation.
+  const convNum = 0;
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
+  const convBodyTerm = uniqueTermGenerator(
     UNIQUE_OFFSET_BODY + UNIQUE_OFFSET_CONV + convNum
   );
   query.bodyMatches(convBodyTerm);
-  queryExpect(
+  await queryExpect(
     query,
     expectFulltextResults ? world.conversationLists[convNum] : []
   );
-  return false; // async pend on queryExpect
 }
 
 /**
  * Test attachment name searching using the conversation unique attachment term.
  *
- * @tests gloda.noun.message.attr.attachmentNamesMatch
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.attachmentNamesMatch
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_attachment_names() {
-  let convNum = 0;
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
-  let convUniqueAttachment = uniqueTermGenerator(
+async function test_query_messages_by_attachment_names() {
+  const convNum = 0;
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
+  const convUniqueAttachment = uniqueTermGenerator(
     UNIQUE_OFFSET_ATTACHMENT + UNIQUE_OFFSET_CONV + convNum
   );
   query.attachmentNamesMatch(convUniqueAttachment);
-  queryExpect(
+  await queryExpect(
     query,
     expectFulltextResults ? world.conversationLists[convNum] : []
   );
-  return false; // async pend on queryExpect
 }
 
 /**
  * Test author name fulltext searching using an arbitrary author.
  *
- * @tests gloda.noun.message.attr.authorMatches
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.authorMatches
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_authorMatches_name() {
-  let [authorName, authorMail] = world.peoples[0];
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_authorMatches_name() {
+  const [authorName, authorMail] = world.peoples[0];
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.authorMatches(authorName);
-  queryExpect(query, world.authorGroups[authorMail]);
-  return false; // async pend on queryExpect
+  await queryExpect(query, world.authorGroups[authorMail]);
 }
 
 /**
  * Test author mail address fulltext searching using an arbitrary author.
  *
- * @tests gloda.noun.message.attr.authorMatches
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.authorMatches
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_authorMatches_email() {
-  let [, authorMail] = world.peoples[0];
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_authorMatches_email() {
+  const [, authorMail] = world.peoples[0];
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.authorMatches(authorMail);
-  queryExpect(query, world.authorGroups[authorMail]);
-  return false; // async pend on queryExpect
+  await queryExpect(query, world.authorGroups[authorMail]);
 }
 
 /**
@@ -571,15 +623,14 @@ function test_query_messages_by_authorMatches_email() {
  *  all 'peoples' messages are sent to all of them, any choice from peoples
  *  gets us all 'peoplesMessages'.
  *
- * @tests gloda.noun.message.attr.recipientsMatch
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.recipientsMatch
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_recipients_name() {
-  let name = world.peoples[0][0];
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_recipients_name() {
+  const name = world.peoples[0][0];
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.recipientsMatch(name);
-  queryExpect(query, world.peoplesMessages);
-  return false; // async pend on queryExpect
+  await queryExpect(query, world.peoplesMessages);
 }
 
 /**
@@ -587,46 +638,44 @@ function test_query_messages_by_recipients_name() {
  *  all 'peoples' messages are sent to all of them, any choice from peoples
  *  gets us all 'peoplesMessages'.
  *
- * @tests gloda.noun.message.attr.recipientsMatch
- * @tests gloda.datastore.sqlgen.kConstraintFulltext
+ * Tests gloda.noun.message.attr.recipientsMatch
+ * Tests gloda.datastore.sqlgen.kConstraintFulltext
  */
-function test_query_messages_by_recipients_email() {
-  let [, mail] = world.peoples[0];
-  let query = Gloda.newQuery(Gloda.NOUN_MESSAGE);
+async function test_query_messages_by_recipients_email() {
+  const [, mail] = world.peoples[0];
+  const query = Gloda.newQuery(GlodaConstants.NOUN_MESSAGE);
   query.recipientsMatch(mail);
-  queryExpect(query, world.peoplesMessages);
-  return false; // async pend on queryExpect
+  await queryExpect(query, world.peoplesMessages);
 }
 
 /* === contacts === */
 
 var contactLikeQuery;
 /**
- * @tests gloda.noun.contact.attr.name
- * @tests gloda.datastore.sqlgen.kConstraintStringLike
+ * Tests gloda.noun.contact.attr.name
+ * Tests gloda.datastore.sqlgen.kConstraintStringLike
  */
-function test_query_contacts_by_name() {
-  // let's use like... we need to test that...
-  contactLikeQuery = Gloda.newQuery(Gloda.NOUN_CONTACT);
-  let personName = world.peoples[0][0];
-  // chop off the first and last letter...  this isn't the most edge-case
+async function test_query_contacts_by_name() {
+  // Let's use like... we need to test that.
+  contactLikeQuery = Gloda.newQuery(GlodaConstants.NOUN_CONTACT);
+  const personName = world.peoples[0][0];
+  // Chop off the first and last letter...  this isn't the most edge-case
   //  handling way to roll, but LOOK OVER THERE? IS THAT ELVIS?
-  let personNameSubstring = personName.substring(1, personName.length - 1);
+  const personNameSubstring = personName.substring(1, personName.length - 1);
   contactLikeQuery.nameLike(
     contactLikeQuery.WILDCARD,
     personNameSubstring,
     contactLikeQuery.WILDCARD
   );
 
-  queryExpect(contactLikeQuery, [personName]);
-  return false; // async pend on queryExpect
+  await queryExpect(contactLikeQuery, [personName]);
 }
 
 /**
- * @tests gloda.query.test.kConstraintStringLike
+ * Tests gloda.query.test.kConstraintStringLike
  */
 function test_query_contacts_by_name_nonmatch() {
-  let otherContact = outlierIdentityCollection.items[0].contact;
+  const otherContact = outlierIdentityCollection.items[0].contact;
   if (contactLikeQuery.test(otherContact)) {
     do_throw("The string LIKE mechanism as applied to contacts does not work.");
   }
@@ -636,24 +685,41 @@ function test_query_contacts_by_name_nonmatch() {
 
 var peoplesIdentityQuery;
 var peoplesIdentityCollection;
-function test_query_identities_for_peoples() {
-  peoplesIdentityQuery = Gloda.newQuery(Gloda.NOUN_IDENTITY);
+async function test_query_identities_for_peoples() {
+  peoplesIdentityQuery = Gloda.newQuery(GlodaConstants.NOUN_IDENTITY);
   peoplesIdentityQuery.kind("email");
-  let peopleAddrs = world.peoples.map(nameAndAddr => nameAndAddr[1]);
+  const peopleAddrs = world.peoples.map(nameAndAddr => nameAndAddr[1]);
   peoplesIdentityQuery.value.apply(peoplesIdentityQuery, peopleAddrs);
-  peoplesIdentityCollection = queryExpect(peoplesIdentityQuery, peopleAddrs);
-  return false; // async pend on queryExpect
+  peoplesIdentityCollection = await queryExpect(
+    peoplesIdentityQuery,
+    peopleAddrs
+  );
 }
 
 var outlierIdentityQuery;
 var outlierIdentityCollection;
-function test_query_identities_for_outliers() {
-  outlierIdentityQuery = Gloda.newQuery(Gloda.NOUN_IDENTITY);
+async function test_query_identities_for_outliers() {
+  outlierIdentityQuery = Gloda.newQuery(GlodaConstants.NOUN_IDENTITY);
   outlierIdentityQuery.kind("email");
-  let outlierAddrs = [world.outlierAuthor[1], world.outlierFriend[1]];
+  const outlierAddrs = [world.outlierAuthor[1], world.outlierFriend[1]];
   outlierIdentityQuery.value.apply(outlierIdentityQuery, outlierAddrs);
-  outlierIdentityCollection = queryExpect(outlierIdentityQuery, outlierAddrs);
-  return false; // async pend on queryExpect
+  outlierIdentityCollection = await queryExpect(
+    outlierIdentityQuery,
+    outlierAddrs
+  );
+}
+
+var maillessIdentityQuery;
+var maillessIdentityCollection;
+async function test_query_identities_for_mailless() {
+  maillessIdentityQuery = Gloda.newQuery(GlodaConstants.NOUN_IDENTITY);
+  maillessIdentityQuery.kind("email");
+  const maillessAddrs = [world.maillessAuthor, world.maillessFriend];
+  maillessIdentityQuery.value.apply(maillessIdentityQuery, maillessAddrs);
+  maillessIdentityCollection = await queryExpect(
+    maillessIdentityQuery,
+    maillessAddrs
+  );
 }
 
 function test_query_identities_by_kind_and_value_nonmatches() {
@@ -663,17 +729,16 @@ function test_query_identities_by_kind_and_value_nonmatches() {
   );
 }
 
-/* ===== Driver ===== */
+function test_sanity_test_environment() {
+  Assert.ok(msgGen, "Sanity that msgGen is set.");
+  Assert.ok(messageInjection, "Sanity that messageInjection is set.");
+}
 
-/* exported tests */
-var tests = [
-  function pre_setup_populate() {
-    pre_setup_populate_hook();
-  },
+var base_query_messages_tests = [
+  test_sanity_test_environment,
+  function pre_setup_populate() {},
   setup_populate,
-  function post_setup_populate() {
-    post_setup_populate_hook();
-  },
+  function post_setup_populate() {},
   test_query_messages_by_conversation,
   test_query_messages_by_conversation,
   test_query_messages_by_conversation_nonmatches,
@@ -682,18 +747,20 @@ var tests = [
   test_query_messages_by_folder_nonmatches,
   test_get_message_for_header,
   test_get_messages_for_headers,
-  // need to do the identity and contact lookups so we can have their results
-  //  for the other message-related queries
+  // Need to do the identity and contact lookups so we can have their results
+  //  for the other message-related queries.
   test_query_identities_for_peoples,
   test_query_identities_for_outliers,
+  test_query_identities_for_mailless,
   test_query_identities_by_kind_and_value_nonmatches,
-  // back to messages!
+  // Back to messages!
   test_query_messages_by_identity_peoples,
   test_query_messages_by_identity_outlier,
+  test_query_messages_by_identity_mailless,
   test_query_messages_by_identity_nonmatches,
   test_query_messages_by_date,
   test_query_messages_by_date_nonmatches,
-  // fulltext
+  // Fulltext
   test_query_messages_by_subject_text,
   test_query_messages_by_body_text,
   test_query_messages_by_attachment_names,
@@ -701,7 +768,7 @@ var tests = [
   test_query_messages_by_authorMatches_email,
   test_query_messages_by_recipients_name,
   test_query_messages_by_recipients_email,
-  // like
+  // Like
   test_query_contacts_by_name,
   test_query_contacts_by_name_nonmatch,
 ];
