@@ -13,17 +13,13 @@
 class nsIImapIncomingServer;
 
 // f4d89e3e-77da-492c-962b-7835f0742c22
-#define NS_IIMAPHOSTSESSIONLIST_IID                  \
-  {                                                  \
-    0xf4d89e3e, 0x77da, 0x492c, {                    \
-      0x96, 0x2b, 0x78, 0x35, 0xf0, 0x74, 0x2c, 0x22 \
-    }                                                \
-  }
+#define NS_IIMAPHOSTSESSIONLIST_IID \
+  {0xf4d89e3e, 0x77da, 0x492c, {0x96, 0x2b, 0x78, 0x35, 0xf0, 0x74, 0x2c, 0x22}}
 
 // this is an interface to a linked list of host info's
 class nsIImapHostSessionList : public nsISupports {
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIMAPHOSTSESSIONLIST_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_IIMAPHOSTSESSIONLIST_IID)
 
   // Host List
   NS_IMETHOD AddHostToList(const char* serverKey,
@@ -76,8 +72,6 @@ class nsIImapHostSessionList : public nsISupports {
                                            bool inProgress) = 0;
   NS_IMETHOD GetDiscoveryForHostInProgress(const char* serverKey,
                                            bool& result) = 0;
-  NS_IMETHOD SetSavedTlsError(const char* serverKey, nsresult errorCode) = 0;
-  NS_IMETHOD GetSavedTlsError(const char* serverKey, nsresult& result) = 0;
 
   // Trash Folder
   NS_IMETHOD SetOnlineTrashFolderExistsForHost(const char* serverKey,
@@ -125,7 +119,4 @@ class nsIImapHostSessionList : public nsISupports {
       const char* serverKey, const char* boxName, char delimiter) = 0;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsIImapHostSessionList,
-                              NS_IIMAPHOSTSESSIONLIST_IID)
-
 #endif
