@@ -3,16 +3,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "msgCore.h"
 #include "nsImapHostSessionList.h"
 #include "nsImapNamespace.h"
 #include "nsIImapIncomingServer.h"
 #include "nsCOMPtr.h"
 #include "nsIMsgIncomingServer.h"
 #include "nsIObserverService.h"
-#include "nsServiceManagerUtils.h"
-#include "nsMsgUtils.h"
 #include "mozilla/Services.h"
+#include "nsString.h"
+#include "plstr.h"
 
 nsIMAPHostInfo::nsIMAPHostInfo(const char* serverKey,
                                nsIImapIncomingServer* server) {
@@ -22,15 +21,8 @@ nsIMAPHostInfo::nsIMAPHostInfo(const char* serverKey,
   fNextHost = NULL;
   fCapabilityFlags = kCapabilityUndefined;
   fHierarchyDelimiters = NULL;
-#ifdef DEBUG_bienvenu1
-  fHaveWeEverDiscoveredFolders =
-      true;  // try this, see what bad happens - we'll need to
-             // figure out a way to make new accounts have it be false
-#else
-  fHaveWeEverDiscoveredFolders = false;  // try this, see what bad happens
-#endif
+  fHaveWeEverDiscoveredFolders = false;
   fDiscoveryForHostInProgress = false;
-  fTlsErrorCode = NS_OK;
   fCanonicalOnlineSubDir = NULL;
   fNamespaceList = nsImapNamespaceList::CreatensImapNamespaceList();
   fUsingSubscription = true;
@@ -316,27 +308,6 @@ NS_IMETHODIMP nsImapHostSessionList::SetDiscoveryForHostInProgress(
   return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
 }
 
-NS_IMETHODIMP nsImapHostSessionList::GetSavedTlsError(const char* serverKey,
-                                                      nsresult& result) {
-  PR_EnterMonitor(gCachedHostInfoMonitor);
-  nsIMAPHostInfo* host = FindHost(serverKey);
-  if (host)
-    result = host->fTlsErrorCode;
-  else
-    result = NS_ERROR_UNEXPECTED;
-  PR_ExitMonitor(gCachedHostInfoMonitor);
-  return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
-}
-
-NS_IMETHODIMP nsImapHostSessionList::SetSavedTlsError(const char* serverKey,
-                                                      nsresult tlsError) {
-  PR_EnterMonitor(gCachedHostInfoMonitor);
-  nsIMAPHostInfo* host = FindHost(serverKey);
-  if (host) host->fTlsErrorCode = tlsError;
-  PR_ExitMonitor(gCachedHostInfoMonitor);
-  return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
-}
-
 NS_IMETHODIMP nsImapHostSessionList::SetOnlineTrashFolderExistsForHost(
     const char* serverKey, bool exists) {
   PR_EnterMonitor(gCachedHostInfoMonitor);
