@@ -6,8 +6,8 @@
 #ifndef __nsImapIncomingServer_h
 #define __nsImapIncomingServer_h
 
-#include "mozilla/Attributes.h"
 #include "msgCore.h"
+#include "nsImapCore.h"
 #include "nsIImapIncomingServer.h"
 #include "nsMsgIncomingServer.h"
 #include "nsIImapServerSink.h"
@@ -18,6 +18,7 @@
 #include "nsCOMArray.h"
 #include "nsTArray.h"
 #include "mozilla/Mutex.h"
+#include "mozilla/Monitor.h"
 
 /* get some implementation from nsMsgIncomingServer */
 class nsImapIncomingServer : public nsMsgIncomingServer,
@@ -44,15 +45,10 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
   NS_IMETHOD PerformBiff(nsIMsgWindow* aMsgWindow) override;
   NS_IMETHOD PerformExpand(nsIMsgWindow* aMsgWindow) override;
   NS_IMETHOD CloseCachedConnections() override;
-  NS_IMETHOD GetConstructedPrettyName(nsAString& retval) override;
+  NS_IMETHOD GetConstructedPrettyName(nsACString& retval) override;
   NS_IMETHOD GetCanBeDefaultServer(bool* canBeDefaultServer) override;
-  NS_IMETHOD GetCanCompactFoldersOnServer(
-      bool* canCompactFoldersOnServer) override;
-  NS_IMETHOD GetCanUndoDeleteOnServer(bool* canUndoDeleteOnServer) override;
   NS_IMETHOD GetCanSearchMessages(bool* canSearchMessages) override;
-  NS_IMETHOD GetCanEmptyTrashOnExit(bool* canEmptyTrashOnExit) override;
   NS_IMETHOD GetOfflineSupportLevel(int32_t* aSupportLevel) override;
-  NS_IMETHOD GeneratePrettyNameForMigration(nsAString& aPrettyName) override;
   NS_IMETHOD GetSupportsDiskSpace(bool* aSupportsDiskSpace) override;
   NS_IMETHOD GetCanCreateFoldersOnServer(
       bool* aCanCreateFoldersOnServer) override;
@@ -62,11 +58,7 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
   NS_IMETHOD GetSearchScope(nsMsgSearchScopeValue* searchScope) override;
   NS_IMETHOD GetServerRequiresPasswordForBiff(
       bool* aServerRequiresPasswordForBiff) override;
-  NS_IMETHOD OnUserOrHostNameChanged(const nsACString& oldName,
-                                     const nsACString& newName,
-                                     bool hostnameChanged) override;
-  NS_IMETHOD GetNumIdleConnections(int32_t* aNumIdleConnections);
-  NS_IMETHOD ForgetSessionPassword() override;
+  NS_IMETHOD ForgetSessionPassword(bool modifyLogin) override;
   NS_IMETHOD GetMsgFolderFromURI(nsIMsgFolder* aFolderResource,
                                  const nsACString& aURI,
                                  nsIMsgFolder** aFolder) override;
@@ -77,14 +69,12 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
  protected:
   virtual ~nsImapIncomingServer();
   nsresult GetFolder(const nsACString& name, nsIMsgFolder** pFolder);
-  virtual nsresult CreateRootFolderFromUri(const nsACString& serverUri,
-                                           nsIMsgFolder** rootFolder) override;
   nsresult ResetFoldersToUnverified(nsIMsgFolder* parentFolder);
   void GetUnverifiedSubFolders(nsIMsgFolder* parentFolder,
                                nsCOMArray<nsIMsgImapMailFolder>& aFoldersArray);
   void GetUnverifiedFolders(nsCOMArray<nsIMsgImapMailFolder>& aFolderArray);
-  bool NoDescendentsAreVerified(nsIMsgFolder* parentFolder);
-  bool AllDescendentsAreNoSelect(nsIMsgFolder* parentFolder);
+  bool NoDescendantsAreVerified(nsIMsgFolder* parentFolder);
+  bool AllDescendantsAreNoSelect(nsIMsgFolder* parentFolder);
 
   nsresult GetStringBundle();
   static nsresult AlertUser(const nsAString& aString, nsIMsgMailNewsUrl* aUrl);
@@ -135,7 +125,12 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
   bool m_shuttingDown;
   bool mUtf8AcceptEnabled;
 
+  // Protect access to Url queue.
   mozilla::Mutex mLock;
+
+  // Only one connection at a time should be a allowed to attempt logon.
+  mozilla::Monitor mLogonMonitor;
+
   // subscribe dialog stuff
   nsresult AddFolderToSubscribeDialog(const char* parentUri, const char* uri,
                                       const char* folderName);
@@ -148,6 +143,9 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
                                 nsACString& folderUriWithNamespace,
                                 bool& namespacePrefixAdded,
                                 bool caseInsensitive, nsIMsgFolder** aFolder);
+
+  // Utility function to obtain the imap (short) path for a folder.
+  static nsresult PathFromFolder(nsIMsgFolder* folder, nsACString& shortPath);
 };
 
 #endif
