@@ -6,7 +6,6 @@
 #define nsImapMailFolder_h__
 
 #include "mozilla/Attributes.h"
-#include "nsImapCore.h"  // so that consumers including ImapMailFolder.h also get the kImapMsg* constants
 #include "nsMsgDBFolder.h"
 #include "nsIImapMailFolderSink.h"
 #include "nsIImapMessageSink.h"
@@ -20,32 +19,24 @@
 #include "nsIMsgFilterList.h"
 #include "prmon.h"
 #include "nsIMsgImapMailFolder.h"
-#include "nsIMsgThread.h"
-#include "nsIImapMailFolderSink.h"
 #include "nsIMsgFilterPlugin.h"
-#include "nsISimpleEnumerator.h"
-#include "nsIStringEnumerator.h"
 #include "nsTHashMap.h"
 #include "nsITimer.h"
-#include "nsCOMArray.h"
 #include "nsAutoSyncState.h"
 
 class nsImapMoveCoalescer;
 class nsIMsgIdentity;
 class nsIMsgOfflineImapOperation;
+class nsIMsgThread;
 
 #define COPY_BUFFER_SIZE 16384
 
-#define NS_IMAPMAILCOPYSTATE_IID                     \
-  {                                                  \
-    0xb64534f0, 0x3d53, 0x11d3, {                    \
-      0xac, 0x2a, 0x00, 0x80, 0x5f, 0x8a, 0xc9, 0x68 \
-    }                                                \
-  }
+#define NS_IMAPMAILCOPYSTATE_IID \
+  {0xb64534f0, 0x3d53, 0x11d3, {0xac, 0x2a, 0x00, 0x80, 0x5f, 0x8a, 0xc9, 0x68}}
 
 class nsImapMailCopyState : public nsISupports {
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMAPMAILCOPYSTATE_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_IMAPMAILCOPYSTATE_IID)
 
   NS_DECL_THREADSAFE_ISUPPORTS
 
@@ -59,6 +50,9 @@ class nsImapMailCopyState : public nsISupports {
                                                    // operation
   nsCOMPtr<nsIFile> m_tmpFile;         // temp file spec for copy operation
   nsCOMPtr<nsIMsgWindow> m_msgWindow;  // msg window for copy operation
+  nsCOMPtr<nsIMsgFolder> m_arrFolder;  // arrived folder (actual folder
+                                       // resulting from moving/copying a
+                                       // folder)
 
   nsCOMPtr<nsIMsgMessageService>
       m_msgService;        // source folder message service; can
@@ -86,8 +80,6 @@ class nsImapMailCopyState : public nsISupports {
   virtual ~nsImapMailCopyState();
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsImapMailCopyState, NS_IMAPMAILCOPYSTATE_IID)
-
 // ACLs for this folder.
 // Generally, we will try to always query this class when performing
 // an operation on the folder.
@@ -225,33 +217,31 @@ class nsImapMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD UpdateFolder(nsIMsgWindow* aWindow) override;
 
-  NS_IMETHOD CreateSubfolder(const nsAString& folderName,
+  NS_IMETHOD CreateSubfolder(const nsACString& folderName,
                              nsIMsgWindow* msgWindow) override;
-  NS_IMETHOD AddSubfolder(const nsAString& aName,
+  NS_IMETHOD AddSubfolder(const nsACString& aName,
                           nsIMsgFolder** aChild) override;
   NS_IMETHODIMP CreateStorageIfMissing(nsIUrlListener* urlListener) override;
 
   NS_IMETHOD Compact(nsIUrlListener* aListener,
                      nsIMsgWindow* aMsgWindow) override;
-  NS_IMETHOD CompactAll(nsIUrlListener* aListener, nsIMsgWindow* aMsgWindow,
-                        bool aCompactOfflineAlso) override;
-  NS_IMETHOD EmptyTrash(nsIMsgWindow* msgWindow,
-                        nsIUrlListener* aListener) override;
+  NS_IMETHOD CompactAll(nsIUrlListener* aListener,
+                        nsIMsgWindow* aMsgWindow) override;
+  NS_IMETHOD EmptyTrash(nsIUrlListener* aListener) override;
   NS_IMETHOD CopyDataToOutputStreamForAppend(
       nsIInputStream* aIStream, int32_t aLength,
       nsIOutputStream* outputStream) override;
   NS_IMETHOD CopyDataDone() override;
   NS_IMETHOD DeleteStorage() override;
-  NS_IMETHOD Rename(const nsAString& newName, nsIMsgWindow* msgWindow) override;
+  NS_IMETHOD Rename(const nsACString& newName,
+                    nsIMsgWindow* msgWindow) override;
   NS_IMETHOD RenameSubFolders(nsIMsgWindow* msgWindow,
                               nsIMsgFolder* oldFolder) override;
   NS_IMETHOD GetNoSelect(bool* aResult) override;
 
-  NS_IMETHOD GetPrettyName(nsAString& prettyName)
+  NS_IMETHOD GetPrettyName(nsACString& prettyName)
       override;  // Override of the base, for top-level mail folder
 
-  NS_IMETHOD GetFolderURL(nsACString& url) override;
-
   NS_IMETHOD UpdateSummaryTotals(bool force) override;
 
   NS_IMETHOD GetDeletable(bool* deletable) override;
@@ -271,11 +261,9 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD MarkMessagesFlagged(const nsTArray<RefPtr<nsIMsgDBHdr>>& messages,
                                  bool markFlagged) override;
   NS_IMETHOD MarkThreadRead(nsIMsgThread* thread) override;
-  NS_IMETHOD SetLabelForMessages(const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
-                                 nsMsgLabelValue aLabel) override;
   NS_IMETHOD SetJunkScoreForMessages(
-      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
-      const nsACString& aJunkScore) override;
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& messages, nsMsgJunkScore junkScore,
+      const nsACString& junkScoreOrigin, int32_t junkPercent) override;
   NS_IMETHOD DeleteSelf(nsIMsgWindow* msgWindow) override;
   NS_IMETHOD ReadFromFolderCacheElem(
       nsIMsgFolderCacheElement* element) override;
@@ -303,9 +291,6 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD GetNewMessages(nsIMsgWindow* aWindow,
                             nsIUrlListener* aListener) override;
 
-  NS_IMETHOD GetFilePath(nsIFile** aPathName) override;
-  NS_IMETHOD SetFilePath(nsIFile* aPath) override;
-
   NS_IMETHOD Shutdown(bool shutdownChildren) override;
 
   NS_IMETHOD DownloadMessagesForOffline(
@@ -317,7 +302,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD GetCanFileMessages(bool* aCanFileMessages) override;
   NS_IMETHOD GetCanDeleteMessages(bool* aCanDeleteMessages) override;
   NS_IMETHOD FetchMsgPreviewText(nsTArray<nsMsgKey> const& aKeysToFetch,
-                                 bool aLocalOnly, nsIUrlListener* aUrlListener,
+                                 nsIUrlListener* aUrlListener,
                                  bool* aAsyncResults) override;
 
   NS_IMETHOD AddKeywordsToMessages(
@@ -327,14 +312,10 @@ class nsImapMailFolder : public nsMsgDBFolder,
       const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
       const nsACString& aKeywords) override;
 
-  NS_IMETHOD NotifyCompactCompleted() override;
-
   // overrides nsMsgDBFolder::HasMsgOffline()
   NS_IMETHOD HasMsgOffline(nsMsgKey msgKey, bool* _retval) override;
-  // overrides nsMsgDBFolder::GetOfflineFileStream()
-  NS_IMETHOD GetOfflineFileStream(nsMsgKey msgKey, int64_t* offset,
-                                  uint32_t* size,
-                                  nsIInputStream** aFileStream) override;
+  NS_IMETHOD GetLocalMsgStream(nsIMsgDBHdr* hdr,
+                               nsIInputStream** stream) override;
 
   NS_DECL_NSIMSGIMAPMAILFOLDER
   NS_DECL_NSIIMAPMAILFOLDERSINK
@@ -353,20 +334,9 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD SetFilterList(nsIMsgFilterList* aMsgFilterList) override;
   NS_IMETHOD GetCustomIdentity(nsIMsgIdentity** aIdentity) override;
 
-  /**
-   * This method is used to locate a folder where a msg could be present, not
-   * just the folder where the message first arrives, this method searches for
-   * the existence of msg in all the folders/labels that we retrieve from
-   * X-GM-LABELS also. overrides nsMsgDBFolder::GetOfflineMsgFolder()
-   *  @param msgKey key  of the msg for which we are trying to get the folder;
-   *  @param aMsgFolder  required folder;
-   */
-  NS_IMETHOD GetOfflineMsgFolder(nsMsgKey msgKey,
-                                 nsIMsgFolder** aMsgFolder) override;
-
   NS_IMETHOD GetIncomingServerType(nsACString& serverType) override;
 
-  nsresult AddSubfolderWithPath(nsAString& name, nsIFile* dbPath,
+  nsresult AddSubfolderWithPath(const nsACString& name, nsIFile* folderPath,
                                 nsIMsgFolder** child, bool brandNew = false);
   nsresult MoveIncorporatedMessage(nsIMsgDBHdr* mailHdr,
                                    nsIMsgDatabase* sourceDB,
@@ -387,15 +357,13 @@ class nsImapMailFolder : public nsMsgDBFolder,
   nsresult SetSupportedUserFlags(uint32_t userFlags);
   nsresult GetSupportedUserFlags(uint32_t* userFlags);
 
-  // Find the start of a range of msgKeys that can hold srcCount headers.
-  nsresult FindOpenRange(nsMsgKey& fakeBase, uint32_t srcCount);
-
  protected:
   virtual ~nsImapMailFolder();
   // Helper methods
-
-  virtual nsresult CreateChildFromURI(const nsACString& uri,
-                                      nsIMsgFolder** folder) override;
+  nsresult CreateFileForDB(const nsAString& userLeafName, nsIFile* baseDir,
+                           nsIFile** dbFile);
+  nsresult ExpungeAndCompact(nsIUrlListener* aListener,
+                             nsIMsgWindow* aMsgWindow);
   void FindKeysToAdd(const nsTArray<nsMsgKey>& existingKeys,
                      nsTArray<nsMsgKey>& keysToFetch, uint32_t& numNewUnread,
                      nsIImapFlagAndUidState* flagState);
@@ -441,7 +409,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   virtual bool DeleteIsMoveToTrash();
   nsresult GetFolder(const nsACString& name, nsIMsgFolder** pFolder);
   nsresult GetTrashFolder(nsIMsgFolder** pTrashFolder);
-  bool TrashOrDescendentOfTrash(nsIMsgFolder* folder);
+  bool TrashOrDescendantOfTrash(nsIMsgFolder* folder);
   static bool ShouldCheckAllFolders(nsIImapIncomingServer* imapServer);
   nsresult GetServerKey(nsACString& serverKey);
   nsresult DisplayStatusMsg(nsIImapUrl* aImapUrl, const nsAString& msg);
@@ -480,9 +448,6 @@ class nsImapMailFolder : public nsMsgDBFolder,
   nsresult GetClearedOriginalOp(nsIMsgOfflineImapOperation* op,
                                 nsIMsgOfflineImapOperation** originalOp,
                                 nsIMsgDatabase** originalDB);
-  nsresult GetOriginalOp(nsIMsgOfflineImapOperation* op,
-                         nsIMsgOfflineImapOperation** originalOp,
-                         nsIMsgDatabase** originalDB);
   MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult CopyMessagesOffline(
       nsIMsgFolder* srcFolder, nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
       bool isMove, nsIMsgWindow* msgWindow,
@@ -494,14 +459,12 @@ class nsImapMailFolder : public nsMsgDBFolder,
                               nsIMsgDBHdr* origHdr, nsIInputStream* inputStream,
                               nsIOutputStream* outputStream);
 
-  void GetTrashFolderName(nsAString& aFolderName);
+  void GetTrashFolderName(nsACString& aFolderName);
   bool ShowPreviewText();
 
   // Pseudo-Offline operation playback timer
   static void PlaybackTimerCallback(nsITimer* aTimer, void* aClosure);
 
-  nsresult CreatePlaybackTimer();
-
   // Allocate and initialize associated auto-sync state object.
   void InitAutoSyncState();
 
@@ -521,6 +484,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   /// the junk destination folder
   nsCOMPtr<nsIMsgFolder> mSpamFolder;
   nsMsgKey m_curMsgUid;
+  nsMsgKey m_previousHighestUid;
   uint32_t m_uidValidity;
 
   // These three vars are used to store counts from STATUS or SELECT command
@@ -556,10 +520,6 @@ class nsImapMailFolder : public nsMsgDBFolder,
   bool m_folderNeedsACLListed;
   bool m_performingBiff;
   bool m_updatingFolder;
-  // These two vars are used to keep track of compaction state so we can know
-  // when to send a done notification.
-  bool m_compactingOfflineStore;
-  bool m_expunging;
   bool m_applyIncomingFilters;  // apply filters to this folder, even if not the
                                 // inbox
   nsMsgIMAPFolderACL* m_folderACL;
@@ -602,5 +562,14 @@ class nsImapMailFolder : public nsMsgDBFolder,
   void DeleteStoreMessages(const nsTArray<nsMsgKey>& aMessages);
   static void DeleteStoreMessages(const nsTArray<nsMsgKey>& aMessages,
                                   nsIMsgFolder* aFolder);
+  /**
+   * This method is used to locate a folder where a msg could be present, not
+   * just the folder where the message first arrives, this method searches for
+   * the existence of msg in all the folders/labels that we retrieve from
+   * X-GM-LABELS also.
+   *  @param msgKey key  of the msg for which we are trying to get the folder;
+   *  @param aMsgFolder  required folder;
+   */
+  nsresult GetOfflineMsgFolder(nsMsgKey msgKey, nsIMsgFolder** aMsgFolder);
 };
 #endif
