@@ -331,7 +331,6 @@ class ProfileImporterController extends ImporterController {
     Thunderbird: "ThunderbirdProfileImporter",
     Seamonkey: "SeamonkeyProfileImporter",
     Outlook: "OutlookProfileImporter",
-    Becky: "BeckyProfileImporter",
     AppleMail: "AppleMailProfileImporter",
   };
 
@@ -343,7 +342,6 @@ class ProfileImporterController extends ImporterController {
     Thunderbird: "thunderbird",
     Seamonkey: "seamonkey",
     Outlook: "outlook",
-    Becky: "becky",
     AppleMail: "apple-mail",
   };
   _sourceAppName = "thunderbird";
@@ -567,7 +565,7 @@ class ProfileImporterController extends ImporterController {
   /**
    * Map of fluent IDs from ImportItems if they differ.
    *
-   * @type {Object<string>}
+   * @type {object}
    */
   _importItemFluentId = {
     addressBooks: "address-books",
@@ -577,7 +575,7 @@ class ProfileImporterController extends ImporterController {
   /**
    * Set checkbox states according to an ImportItems object.
    *
-   * @param {ImportItems} items.
+   * @param {ImportItems} items
    */
   _setItemsChecked(items) {
     for (const [id, field] of Object.entries(this._itemCheckboxes)) {
@@ -632,7 +630,7 @@ class ProfileImporterController extends ImporterController {
     }
     document.getElementById("appSummaryItems").replaceChildren(
       ...Object.entries(this._getItemsChecked(true))
-        .filter(([item, checked]) => checked)
+        .filter(([, checked]) => checked)
         .map(([item]) => {
           const li = document.createElement("li");
           const fluentId = this._importItemFluentId[item] ?? item;
@@ -697,15 +695,28 @@ class ProfileImporterController extends ImporterController {
   }
 
   async startImport() {
+    const gleanData = {
+      importer: this._importer.NAME,
+      types: Object.entries(this._getItemsChecked())
+        .filter(entry => entry[1])
+        .map(entry => entry[0])
+        .join(","),
+    };
     this.showProgress("progress-pane-importing2");
     if (this._importingFromZip) {
+      gleanData.importer += ",zip";
       this._extractedFileCount = 0;
       try {
         await this._extractZipFile();
       } catch (e) {
         this.showError("error-message-extract-zip-file-failed2");
+        Glean.mail.import.record({ ...gleanData, result: "unzipFailed" });
         throw e;
       }
+    } else if (this._sourceProfile.name) {
+      gleanData.importer += ",profile";
+    } else {
+      gleanData.importer += ",directory";
     }
     this._importer.onProgress = (current, total) => {
       this.updateProgress(
@@ -713,14 +724,15 @@ class ProfileImporterController extends ImporterController {
       );
     };
     try {
-      this.finish(
-        await this._importer.startImport(
-          this._sourceProfile.dir,
-          this._getItemsChecked()
-        )
+      const restartNeeded = await this._importer.startImport(
+        this._sourceProfile.dir,
+        this._getItemsChecked()
       );
+      Glean.mail.import.record({ ...gleanData, result: "succeeded" });
+      this.finish(restartNeeded);
     } catch (e) {
       this.showError("error-message-failed");
+      Glean.mail.import.record({ ...gleanData, result: "failed" });
       throw e;
     } finally {
       if (this._importingFromZip) {
@@ -892,7 +904,7 @@ class AddrBookImporterController extends ImporterController {
     const elList = document.getElementById("directoryList");
     elList.innerHTML = "";
     this._directories = MailServices.ab.directories.filter(
-      dir => dir.dirType == Ci.nsIAbManager.JS_DIRECTORY_TYPE
+      dir => !dir.readOnly
     );
     for (const directory of this._directories) {
       const label = document.createElement("label");
@@ -982,8 +994,18 @@ class AddrBookImporterController extends ImporterController {
       this.finish(
         await this._importer.startImport(this._sourceFile, targetDirectory)
       );
+      Glean.mail.import.record({
+        importer: "addrbook",
+        types: this._fileType,
+        result: "succeeded",
+      });
     } catch (e) {
       this.showError("error-message-failed");
+      Glean.mail.import.record({
+        importer: "addrbook",
+        types: this._fileType,
+        result: "failed",
+      });
       throw e;
     }
   }
@@ -996,6 +1018,9 @@ class AddrBookImporterController extends ImporterController {
 class CalendarImporterController extends ImporterController {
   constructor() {
     super("tabPane-calendar", "calendar");
+    const filter = document.getElementById("calendarFilter");
+    filter.addEventListener("autocomplete", this.onFilterChange.bind(this));
+    filter.addEventListener("search", event => event.preventDefault());
   }
 
   next() {
@@ -1021,16 +1046,67 @@ class CalendarImporterController extends ImporterController {
   }
 
   /**
-   * When filter changes, re-render the item list.
+   * When filter changes, re-render the item list. This function wraps
+   * #onFilterChange in a timer, to reduce the frequency of list updates.
    *
-   * @param {HTMLInputElement} filterInput - The filter input.
+   * @param {Event} event - The "autocomplete" event fired by the filter input.
    */
-  onFilterChange(filterInput) {
-    const term = filterInput.value.toLowerCase();
+  onFilterChange(event) {
+    let searchString = event.detail.trim();
+    if (!searchString) {
+      this._filteredItems = [...this._items];
+      for (const item of this._items) {
+        const element = this._itemElements[item.id];
+        element.hidden = false;
+      }
+      return;
+    }
+
+    searchString = searchString.toLowerCase().normalize();
+
+    // Split the search string into tokens. Quoted strings are preserved.
+    let searchTokens = [];
+    let startIndex;
+    while ((startIndex = searchString.indexOf('"')) != -1) {
+      let endIndex = searchString.indexOf('"', startIndex + 1);
+      if (endIndex == -1) {
+        endIndex = searchString.length;
+      }
+
+      searchTokens.push(searchString.slice(startIndex + 1, endIndex));
+      let query = searchString.slice(0, startIndex);
+      if (endIndex < searchString.length) {
+        query += searchString.slice(endIndex + 1);
+      }
+
+      searchString = query.trim();
+    }
+
+    if (searchString.length != 0) {
+      searchTokens = searchTokens.concat(searchString.split(/\s+/));
+    }
+
     this._filteredItems = [];
+
     for (const item of this._items) {
+      const title = item.title.toLowerCase().normalize();
+      let description;
+      const matches = searchTokens.every(term => {
+        if (title?.includes(term)) {
+          return true;
+        }
+
+        if (description === undefined) {
+          description = item
+            .getProperty("description")
+            ?.toLowerCase()
+            .normalize();
+        }
+        return description?.includes(term);
+      });
+
       const element = this._itemElements[item.id];
-      if (item.title.toLowerCase().includes(term)) {
+      if (matches) {
         element.hidden = false;
         this._filteredItems.push(item);
       } else {
@@ -1074,6 +1150,7 @@ class CalendarImporterController extends ImporterController {
       3
     );
     this.showPane("sources");
+    document.getElementById("calendarNextButton").disabled = false;
   }
 
   /**
@@ -1095,11 +1172,21 @@ class CalendarImporterController extends ImporterController {
       return;
     }
 
-    this._sourceFile = filePicker.file;
+    this.useFile(filePicker.file);
+  }
+
+  /**
+   * Use `file` as the source of items to be imported. Normally the file comes
+   * from the file picker, but it could be given as command-line argument when
+   * starting Thunderbird.
+   *
+   * @param {nsIFile} file
+   */
+  useFile(file) {
+    this._sourceFile = file;
     this._importer = new CalendarFileImporter();
 
-    document.getElementById("calendarSourcePath").textContent =
-      filePicker.file.path;
+    document.getElementById("calendarSourcePath").textContent = file.path;
 
     this._showItems();
   }
@@ -1124,6 +1211,7 @@ class CalendarImporterController extends ImporterController {
     this.showPane("items");
 
     // Give the UI a chance to render.
+    await document.l10n.translateElements([elItemList]);
     await new Promise(resolve => setTimeout(resolve, 100));
 
     try {
@@ -1135,12 +1223,17 @@ class CalendarImporterController extends ImporterController {
 
     document.getElementById("calendarItemsTools").hidden =
       this._items.length < 2;
-    elItemList.innerHTML = "";
+    delete elItemList.dataset.l10nId;
+    elItemList.replaceChildren();
     this._filteredItems = this._items;
     this._selectedItems = new Set(this._items);
     this._itemElements = {};
 
     for (const item of this._items) {
+      if (!item.id) {
+        item.id = Services.uuid.generateUUID().toString().slice(1, 37);
+      }
+
       const wrapper = document.createElement("div");
       wrapper.className = "calendar-item-wrapper";
       elItemList.appendChild(wrapper);
@@ -1233,6 +1326,7 @@ class CalendarImporterController extends ImporterController {
       elList.append(label);
     }
     document.querySelector("input[name=targetCalendar]").checked = true;
+    document.getElementById("calendarNextButton").disabled = false;
 
     this.showPane("calendars");
   }
@@ -1299,9 +1393,11 @@ class CalendarImporterController extends ImporterController {
         [...this._selectedItems],
         targetCalendar
       );
+      Glean.mail.import.record({ importer: "calendar", result: "succeeded" });
       this.finish();
     } catch (e) {
       this.showError("error-message-failed");
+      Glean.mail.import.record({ importer: "calendar", result: "failed" });
       throw e;
     }
   }
@@ -1392,7 +1488,7 @@ class StartController extends ImporterController {
       {
         returnTo: () => {
           this.reset();
-          showTab("tab-start");
+          showTab("start");
           //showTab will always call showInitialStep
         },
       },
@@ -1416,7 +1512,7 @@ class StartController extends ImporterController {
         break;
       default:
         await profileController._onSelectSource(checkedInput.value);
-        showTab("tab-app");
+        showTab("app");
         // Don't change back button state, since we switch to app flow.
         return;
     }
@@ -1429,7 +1525,7 @@ class StartController extends ImporterController {
       {
         returnTo: () => {
           this.reset();
-          showTab("tab-start");
+          showTab("start");
           this._showFile();
         },
       },
@@ -1449,38 +1545,41 @@ class StartController extends ImporterController {
         await profileController._onSelectSource("Thunderbird");
         document.getElementById("appFilePickerZip").checked = true;
         await profileController._onSelectProfile();
-        showTab("tab-app");
+        showTab("app");
         break;
       case "calendar":
         calendarController.reset();
-        showTab("tab-calendar");
+        showTab("calendar");
         calendarController.showInitialStep();
         await calendarController._onSelectSource();
         break;
       case "addressbook":
         addrBookController.reset();
-        showTab("tab-addressBook");
+        showTab("addressBook");
         addrBookController.showInitialStep();
         break;
     }
   }
 }
 
+let currentTab;
+
 /**
  * Show a specific importing tab.
  *
- * @param {"tab-app"|"tab-addressBook"|"tab-calendar"|"tab-export"|"tab-start"} tabId -
+ * @param {"app"|"addressBook"|"calendar"|"export"|"start"} paneId -
  *  Tab to show.
  * @param {boolean} [reset=false] - If the state should be reset as if this was
  *  the initial tab shown.
  */
-function showTab(tabId, reset = false) {
+function showTab(paneId, reset = false) {
   if (reset) {
     Steps.reset();
     restart();
   }
-  const selectedPaneId = `tabPane-${tabId.split("-")[1]}`;
-  const isExport = tabId === "tab-export";
+  currentTab = paneId;
+  const selectedPaneId = `tabPane-${paneId}`;
+  const isExport = paneId === "export";
   document.getElementById("importDocs").hidden = isExport;
   document.getElementById("exportDocs").hidden = !isExport;
   Steps.toggle(!isExport);
@@ -1491,24 +1590,25 @@ function showTab(tabId, reset = false) {
   document.querySelector("link[rel=icon]").href = isExport
     ? "chrome://messenger/skin/icons/new/compact/export.svg"
     : "chrome://messenger/skin/icons/new/compact/import.svg";
-  location.hash = isExport ? "export" : "";
+  location.hash = paneId;
   for (const tabPane of document.querySelectorAll("[id^=tabPane-]")) {
     tabPane.hidden = tabPane.id != selectedPaneId;
   }
-  for (const el of document.querySelectorAll("[id^=tab-]")) {
-    el.classList.toggle("is-selected", el.id == tabId);
-  }
   if (!Steps.hasStepHistory()) {
-    switch (tabId) {
-      case "tab-start":
+    switch (paneId) {
+      case "start":
         startController.showInitialStep();
         break;
-      case "tab-addressBook":
+      case "addressBook":
         addrBookController.showInitialStep();
         break;
-      case "tab-calendar":
+      case "calendar":
         calendarController.showInitialStep();
         break;
+      case "app":
+        // Profile import can't be restored to - app selection is in start flow.
+        showTab("start", true);
+        break;
       default:
     }
   }
@@ -1527,7 +1627,7 @@ function restart() {
 
 let profileController;
 let addrBookController;
-let calendarController;
+var calendarController;
 let exportController;
 let startController;
 
@@ -1537,5 +1637,11 @@ document.addEventListener("DOMContentLoaded", () => {
   calendarController = new CalendarImporterController();
   exportController = new ExportController();
   startController = new StartController();
-  showTab(location.hash === "#export" ? "tab-export" : "tab-start", true);
+  showTab(location.hash ? location.hash.slice(1) : "start", true);
+});
+window.addEventListener("hashchange", () => {
+  const requestedTab = location.hash.slice(1);
+  if (requestedTab !== currentTab) {
+    showTab(requestedTab, true);
+  }
 });
