@@ -11,21 +11,16 @@
 #include "nscore.h"
 #include "nsMsgUtils.h"
 #include "nsIImportService.h"
-#include "nsIImportFieldMap.h"
 #include "nsIImportMailboxDescriptor.h"
 #include "nsIImportABDescriptor.h"
-#include "nsOutlookStringBundle.h"
-#include "nsAbBaseCID.h"
 #include "nsIAbCard.h"
-#include "mdb.h"
 #include "ImportDebug.h"
 #include "nsOutlookMail.h"
-#include "nsUnicharUtils.h"
 #include "nsIOutputStream.h"
+#include "nsIMsgDatabase.h"
 #include "nsIMsgPluggableStore.h"
 #include "nsIMsgHdr.h"
 #include "nsIMsgFolder.h"
-#include "nsMsgI18N.h"
 #include "nsNetUtil.h"
 
 /* ------------ Address book stuff ----------------- */
@@ -337,7 +332,7 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
   mCaller->OpenMessageStore(pFolder);
   if (!mCaller->m_lpMdb) {
     IMPORT_LOG1("*** Unable to obtain mapi message store for mailbox: %S\n",
-                mName);
+                (const wchar_t*)mName);
     mResult = NS_ERROR_FAILURE;
     return NS_OK;  // Sync runnable must return OK.
   }
@@ -356,26 +351,36 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
   ULONG totalCount;
   double doneCalc;
 
-  nsCOMPtr<nsIOutputStream> outputStream;
   nsCOMPtr<nsIMsgPluggableStore> msgStore;
   nsresult rv = mDstFolder->GetMsgStore(getter_AddRefs(msgStore));
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG1("*** Error getting msgStore for mailbox: %S\n",
+                (const wchar_t*)mName);
+    mResult = rv;
+    return NS_OK;  // Sync runnable must return OK.
+  }
+  nsCOMPtr<nsIMsgDatabase> db;
+  rv = mDstFolder->GetMsgDatabase(getter_AddRefs(db));
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG1("*** Error getting DB for mailbox: %S\n",
+                (const wchar_t*)mName);
+    mResult = rv;
+    return NS_OK;  // Sync runnable must return OK.
+  }
 
   while (!done) {
     if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1("*** Error iterating mailbox: %S\n", mName);
+      IMPORT_LOG1("*** Error iterating mailbox: %S\n", (const wchar_t*)mName);
       mResult = NS_ERROR_FAILURE;
       return NS_OK;  // Sync runnable must return OK.
     }
 
-    nsCOMPtr<nsIMsgDBHdr> msgHdr;
-    bool reusable;
-
-    rv = msgStore->GetNewMsgOutputStream(mDstFolder, getter_AddRefs(msgHdr),
-                                         &reusable,
+    nsCOMPtr<nsIOutputStream> outputStream;
+    rv = msgStore->GetNewMsgOutputStream(mDstFolder,
                                          getter_AddRefs(outputStream));
     if (NS_FAILED(rv)) {
-      IMPORT_LOG1("*** Error getting nsIOutputStream of mailbox: %S\n", mName);
+      IMPORT_LOG1("*** Error getting nsIOutputStream of mailbox: %S\n",
+                  (const wchar_t*)mName);
       mResult = rv;
       return NS_OK;  // Sync runnable must return OK.
     }
@@ -391,7 +396,8 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
     if (!done && (oType == MAPI_MESSAGE)) {
       if (!mCaller->m_mapi.OpenMdbEntry(mCaller->m_lpMdb, cbEid, lpEid,
                                         (LPUNKNOWN*)&lpMsg)) {
-        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", mName);
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                    (const wchar_t*)mName);
         mResult = NS_ERROR_FAILURE;
         return NS_OK;  // Sync runnable must return OK.
       }
@@ -407,16 +413,24 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
       rv = ImportMessage(lpMsg, outputStream, mode);
       if (NS_SUCCEEDED(rv)) {  // No errors & really imported
         (*mMsgCount)++;
-        msgStore->FinishNewMessage(outputStream, msgHdr);
+        nsCOMPtr<nsIMsgDBHdr> msgHdr;
+        rv = db->CreateNewHdr(nsMsgKey_None, getter_AddRefs(msgHdr));
+        if (NS_FAILED(rv)) {
+          IMPORT_LOG1("*** Error creating DB header for mailbox: %S\n",
+                      (const wchar_t*)mName);
+          mResult = rv;
+          return NS_OK;  // Sync runnable must return OK.
+        }
+        nsAutoCString storeToken;
+        msgStore->FinishNewMessage(mDstFolder, outputStream, storeToken);
+        msgHdr->SetStoreToken(storeToken);
       } else {
-        IMPORT_LOG1("*** Error reading message from mailbox: %S\n", mName);
-        msgStore->DiscardNewMessage(outputStream, msgHdr);
+        IMPORT_LOG1("*** Error reading message from mailbox: %S\n",
+                    (const wchar_t*)mName);
+        msgStore->DiscardNewMessage(mDstFolder, outputStream);
       }
-      if (!reusable) outputStream->Close();
     }
   }
-
-  if (outputStream) outputStream->Close();
   return NS_OK;
 }
 
@@ -425,7 +439,9 @@ nsresult ProxyImportMailbox(uint32_t* pDoneSoFar, bool* pAbort, int32_t index,
                             int32_t* pMsgCount, nsOutlookMail* aCaller) {
   RefPtr<ImportMailboxRunnable> importMailbox = new ImportMailboxRunnable(
       pDoneSoFar, pAbort, index, pName, dstFolder, pMsgCount, aCaller);
-  nsresult rv = NS_DispatchToMainThread(importMailbox, NS_DISPATCH_SYNC);
+  nsresult rv = NS_DispatchAndSpinEventLoopUntilComplete(
+      "ProxyImportMailbox"_ns, mozilla::GetMainThreadSerialEventTarget(),
+      do_AddRef(importMailbox));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return importMailbox->mResult;
@@ -471,7 +487,7 @@ nsresult ImportMailboxRunnable::ImportMessage(LPMESSAGE lpMsg,
 }
 
 BOOL nsOutlookMail::WriteData(nsIOutputStream* pDest, const char* pData,
-                              int32_t len) {
+                              uint32_t len) {
   uint32_t written;
   nsresult rv = pDest->Write(pData, len, &written);
   return NS_SUCCEEDED(rv) && written == len;
@@ -511,7 +527,7 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
   if (!m_lpMdb) {
     IMPORT_LOG1(
         "*** Unable to obtain mapi message store for address book: %S\n",
-        pName);
+        (const wchar_t*)pName);
     return NS_ERROR_FAILURE;
   }
 
@@ -519,14 +535,6 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
 
   nsresult rv;
 
-  nsCOMPtr<nsIImportFieldMap> pFieldMap;
-
-  nsCOMPtr<nsIImportService> impSvc(
-      do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    rv = impSvc->CreateNewFieldMap(getter_AddRefs(pFieldMap));
-  }
-
   CMapiFolderContents contents(m_lpMdb, pFolder->GetCBEntryID(),
                                pFolder->GetEntryID());
 
@@ -543,7 +551,8 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
     (*pCount)++;
 
     if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1("*** Error iterating address book: %S\n", pName);
+      IMPORT_LOG1("*** Error iterating address book: %S\n",
+                  (const wchar_t*)pName);
       return NS_ERROR_FAILURE;
     }
 
@@ -551,7 +560,8 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
 
     if (!done && (oType == MAPI_MESSAGE)) {
       if (!m_mapi.OpenMdbEntry(m_lpMdb, cbEid, lpEid, (LPUNKNOWN*)&lpMsg)) {
-        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName);
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                    (const wchar_t*)pName);
         return NS_ERROR_FAILURE;
       }
 
@@ -568,10 +578,9 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
           if (pVal) m_mapi.GetStringFromProp(pVal, subject);
 
           nsCOMPtr<nsIAbCard> newCard =
-              do_CreateInstance(NS_ABCARDPROPERTY_CONTRACTID, &rv);
+              do_CreateInstance("@mozilla.org/addressbook/cardproperty;1", &rv);
           if (newCard) {
-            if (BuildCard(subject.get(), pDirectory, newCard, lpMsg,
-                          pFieldMap)) {
+            if (BuildCard(subject.get(), pDirectory, newCard, lpMsg)) {
               nsIAbCard* outCard;
               pDirectory->AddCard(newCard, &outCard);
             }
@@ -581,7 +590,7 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
           subject.Truncate();
           pVal = m_mapi.GetMapiProperty(lpMsg, PR_SUBJECT);
           if (pVal) m_mapi.GetStringFromProp(pVal, subject);
-          CreateList(subject, pDirectory, lpMsg, pFieldMap);
+          CreateList(subject, pDirectory, lpMsg);
         }
       }
 
@@ -593,8 +602,7 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
 }
 nsresult nsOutlookMail::CreateList(const nsString& pName,
                                    nsIAbDirectory* pDirectory,
-                                   LPMAPIPROP pUserList,
-                                   nsIImportFieldMap* pFieldMap) {
+                                   LPMAPIPROP pUserList) {
   // If no name provided then we're done.
   if (pName.IsEmpty()) return NS_OK;
 
@@ -603,7 +611,7 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
   if (!pDirectory) return rv;
 
   nsCOMPtr<nsIAbDirectory> newList =
-      do_CreateInstance(NS_ABDIRPROPERTY_CONTRACTID, &rv);
+      do_CreateInstance("@mozilla.org/addressbook/directoryproperty;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = newList->SetDirName(pName);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -643,7 +651,8 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
     cbEid = sa->lpbin[idx].cb;
 
     if (!m_mapi.OpenEntry(cbEid, lpEid, (LPUNKNOWN*)&lpMsg)) {
-      IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName.get());
+      IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                  static_cast<const wchar_t*>(pName.get()));
       m_mapi.MAPIFreeBuffer(value);
       return NS_ERROR_FAILURE;
     }
@@ -653,9 +662,9 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
     if (pVal) m_mapi.GetStringFromProp(pVal, subject);
 
     nsCOMPtr<nsIAbCard> newCard =
-        do_CreateInstance(NS_ABCARDPROPERTY_CONTRACTID, &rv);
+        do_CreateInstance("@mozilla.org/addressbook/cardproperty;1", &rv);
     if (newCard) {
-      if (BuildCard(subject.get(), pDirectory, newCard, lpMsg, pFieldMap)) {
+      if (BuildCard(subject.get(), pDirectory, newCard, lpMsg)) {
         nsIAbCard* outCard;
         newList->AddCard(newCard, &outCard);
       }
@@ -670,12 +679,12 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
 
 void nsOutlookMail::SanitizeValue(nsString& val) {
   val.ReplaceSubstring(u"\r\n"_ns, u", "_ns);
-  val.ReplaceChar("\r\n", ',');
+  val.ReplaceChar(u"\r\n", u',');
 }
 
 void nsOutlookMail::SplitString(nsString& val1, nsString& val2) {
   // Find the last line if there is more than one!
-  int32_t idx = val1.RFind("\x0D\x0A");
+  int32_t idx = val1.RFind(u"\x0D\x0A");
   int32_t cnt = 2;
   if (idx == -1) {
     cnt = 1;
@@ -690,8 +699,7 @@ void nsOutlookMail::SplitString(nsString& val1, nsString& val2) {
 }
 
 bool nsOutlookMail::BuildCard(const char16_t* pName, nsIAbDirectory* pDirectory,
-                              nsIAbCard* newCard, LPMAPIPROP pUser,
-                              nsIImportFieldMap* pFieldMap) {
+                              nsIAbCard* newCard, LPMAPIPROP pUser) {
   nsString lastName;
   nsString firstName;
   nsString eMail;
@@ -794,30 +802,24 @@ bool nsOutlookMail::BuildCard(const char16_t* pName, nsIAbDirectory* pDirectory,
   nsString value;
   nsString line2;
 
-  if (pFieldMap) {
-    int max = sizeof(gMapiFields) / sizeof(MAPIFields);
-    for (int i = 0; i < max; i++) {
-      pProp = m_mapi.GetMapiProperty(pUser, gMapiFields[i].mapiTag);
-      if (pProp) {
-        m_mapi.GetStringFromProp(pProp, value);
-        if (!value.IsEmpty()) {
-          if (gMapiFields[i].multiLine == kNoMultiLine) {
-            SanitizeValue(value);
-            pFieldMap->SetFieldValue(pDirectory, newCard,
-                                     gMapiFields[i].mozField, value);
-          } else if (gMapiFields[i].multiLine == kIsMultiLine) {
-            pFieldMap->SetFieldValue(pDirectory, newCard,
-                                     gMapiFields[i].mozField, value);
-          } else {
-            line2.Truncate();
-            SplitString(value, line2);
-            if (!value.IsEmpty())
-              pFieldMap->SetFieldValue(pDirectory, newCard,
-                                       gMapiFields[i].mozField, value);
-            if (!line2.IsEmpty())
-              pFieldMap->SetFieldValue(pDirectory, newCard,
-                                       gMapiFields[i].multiLine, line2);
-          }
+  int max = sizeof(gMapiFields) / sizeof(MAPIFields);
+  for (int i = 0; i < max; i++) {
+    pProp = m_mapi.GetMapiProperty(pUser, gMapiFields[i].mapiTag);
+    if (pProp) {
+      m_mapi.GetStringFromProp(pProp, value);
+      if (!value.IsEmpty()) {
+        if (gMapiFields[i].multiLine == kNoMultiLine) {
+          SanitizeValue(value);
+          SetFieldValue(newCard, gMapiFields[i].mozField, value);
+        } else if (gMapiFields[i].multiLine == kIsMultiLine) {
+          SetFieldValue(newCard, gMapiFields[i].mozField, value);
+        } else {
+          line2.Truncate();
+          SplitString(value, line2);
+          if (!value.IsEmpty())
+            SetFieldValue(newCard, gMapiFields[i].mozField, value);
+          if (!line2.IsEmpty())
+            SetFieldValue(newCard, gMapiFields[i].multiLine, line2);
         }
       }
     }
@@ -825,3 +827,127 @@ bool nsOutlookMail::BuildCard(const char16_t* pName, nsIAbDirectory* pDirectory,
 
   return true;
 }
+
+nsresult nsOutlookMail::SetFieldValue(nsIAbCard* row, int32_t fieldNum,
+                                      const nsAString& value) {
+  nsresult rv;
+
+  switch (fieldNum) {
+    case 0:
+      rv = row->SetFirstName(value);
+      break;
+    case 1:
+      rv = row->SetLastName(value);
+      break;
+    case 2:
+      rv = row->SetDisplayName(value);
+      break;
+    case 3:
+      rv = row->SetPropertyAsAString(kNicknameProperty, value);
+      break;
+    case 4:
+      rv = row->SetPrimaryEmail(value);
+      break;
+    case 5:
+      rv = row->SetPropertyAsAString(k2ndEmailProperty, value);
+      break;
+    case 6:
+      rv = row->SetPropertyAsAString(kWorkPhoneProperty, value);
+      break;
+    case 7:
+      rv = row->SetPropertyAsAString(kHomePhoneProperty, value);
+      break;
+    case 8:
+      rv = row->SetPropertyAsAString(kFaxProperty, value);
+      break;
+    case 9:
+      rv = row->SetPropertyAsAString(kPagerProperty, value);
+      break;
+    case 10:
+      rv = row->SetPropertyAsAString(kCellularProperty, value);
+      break;
+    case 11:
+      rv = row->SetPropertyAsAString(kHomeAddressProperty, value);
+      break;
+    case 12:
+      rv = row->SetPropertyAsAString(kHomeAddress2Property, value);
+      break;
+    case 13:
+      rv = row->SetPropertyAsAString(kHomeCityProperty, value);
+      break;
+    case 14:
+      rv = row->SetPropertyAsAString(kHomeStateProperty, value);
+      break;
+    case 15:
+      rv = row->SetPropertyAsAString(kHomeZipCodeProperty, value);
+      break;
+    case 16:
+      rv = row->SetPropertyAsAString(kHomeCountryProperty, value);
+      break;
+    case 17:
+      rv = row->SetPropertyAsAString(kWorkAddressProperty, value);
+      break;
+    case 18:
+      rv = row->SetPropertyAsAString(kWorkAddress2Property, value);
+      break;
+    case 19:
+      rv = row->SetPropertyAsAString(kWorkCityProperty, value);
+      break;
+    case 20:
+      rv = row->SetPropertyAsAString(kWorkStateProperty, value);
+      break;
+    case 21:
+      rv = row->SetPropertyAsAString(kWorkZipCodeProperty, value);
+      break;
+    case 22:
+      rv = row->SetPropertyAsAString(kWorkCountryProperty, value);
+      break;
+    case 23:
+      rv = row->SetPropertyAsAString(kJobTitleProperty, value);
+      break;
+    case 24:
+      rv = row->SetPropertyAsAString(kDepartmentProperty, value);
+      break;
+    case 25:
+      rv = row->SetPropertyAsAString(kCompanyProperty, value);
+      break;
+    case 26:
+      rv = row->SetPropertyAsAString(kWorkWebPageProperty, value);
+      break;
+    case 27:
+      rv = row->SetPropertyAsAString(kHomeWebPageProperty, value);
+      break;
+    case 28:
+      rv = row->SetPropertyAsAString(kBirthYearProperty, value);
+      break;
+    case 29:
+      rv = row->SetPropertyAsAString(kBirthMonthProperty, value);
+      break;
+    case 30:
+      rv = row->SetPropertyAsAString(kBirthDayProperty, value);
+      break;
+    case 31:
+      rv = row->SetPropertyAsAString(kCustom1Property, value);
+      break;
+    case 32:
+      rv = row->SetPropertyAsAString(kCustom2Property, value);
+      break;
+    case 33:
+      rv = row->SetPropertyAsAString(kCustom3Property, value);
+      break;
+    case 34:
+      rv = row->SetPropertyAsAString(kCustom4Property, value);
+      break;
+    case 35:
+      rv = row->SetPropertyAsAString(kNotesProperty, value);
+      break;
+    case 36:
+      rv = row->SetPropertyAsAString(kAIMProperty, value);
+      break;
+    default:
+      rv = NS_ERROR_FAILURE;
+      break;
+  }
+
+  return rv;
+}
