@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,11 +11,7 @@
 #include "nsNewsUtils.h"
 #include "nsMsgUtils.h"
 
-#include "nntpCore.h"
-
 #include "nsCOMPtr.h"
-#include "nsMsgDBCID.h"
-#include "nsMsgNewsCID.h"
 #include "nsIMsgFolder.h"
 #include "nsIMsgNewsFolder.h"
 #include "nsINntpService.h"
@@ -24,13 +20,13 @@
 #include "nsServiceManagerUtils.h"
 
 nsNntpUrl::nsNntpUrl() {
-  m_newsgroupPost = nullptr;
   m_newsAction = nsINntpUrl::ActionUnknown;
   m_addDummyEnvelope = false;
   m_canonicalLineEnding = false;
   m_filePath = nullptr;
   m_getOldMessages = false;
   m_key = nsMsgKey_None;
+  mAutodetectCharset = false;
 }
 
 nsNntpUrl::~nsNntpUrl() {}
@@ -66,7 +62,7 @@ NS_INTERFACE_MAP_END_INHERITING(nsMsgMailNewsUrl)
  * canonicalization.
  */
 
-nsresult nsNntpUrl::SetSpecInternal(const nsACString &aSpec) {
+nsresult nsNntpUrl::SetSpecInternal(const nsACString& aSpec) {
   // For [s]news: URIs, we need to munge the spec if it is no authority, because
   // the URI parser guesses the wrong thing otherwise
   nsCString parseSpec(aSpec);
@@ -100,7 +96,7 @@ nsresult nsNntpUrl::SetSpecInternal(const nsACString &aSpec) {
     nsAutoCString spec;
     rv = GetSpec(spec);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = nsParseNewsMessageURI(spec.get(), m_group, &m_key);
+    rv = nsParseNewsMessageURI(spec, m_group, &m_key);
     NS_ENSURE_SUCCESS(rv, NS_ERROR_MALFORMED_URI);
   } else
     return NS_ERROR_MALFORMED_URI;
@@ -192,12 +188,11 @@ nsresult nsNntpUrl::DetermineNewsAction() {
     m_newsAction = nsINntpUrl::ActionListNewGroups;
     return NS_OK;
   }
-  if (StringBeginsWith(query, NS_LITERAL_CSTRING("search"))) {
+  if (StringBeginsWith(query, "search"_ns)) {
     m_newsAction = nsINntpUrl::ActionSearch;
     return NS_OK;
   }
-  if (StringBeginsWith(query, NS_LITERAL_CSTRING("part=")) ||
-      query.Find("&part=") > 0) {
+  if (StringBeginsWith(query, "part="_ns) || query.Find("&part=") > 0) {
     // news://news.mozilla.org:119/3B98D201.3020100%40cs.com?part=1
     // news://news.mozilla.org:119/b58dme%24aia2%40ripley.netscape.com?header=print&part=1.2&type=image/jpeg&filename=Pole.jpg
     m_newsAction = nsINntpUrl::ActionFetchPart;
@@ -230,13 +225,13 @@ NS_IMETHODIMP nsNntpUrl::SetGetOldMessages(bool aGetOldMessages) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetGetOldMessages(bool *aGetOldMessages) {
+NS_IMETHODIMP nsNntpUrl::GetGetOldMessages(bool* aGetOldMessages) {
   NS_ENSURE_ARG(aGetOldMessages);
   *aGetOldMessages = m_getOldMessages;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetNewsAction(nsNewsAction *aNewsAction) {
+NS_IMETHODIMP nsNntpUrl::GetNewsAction(nsNewsAction* aNewsAction) {
   if (aNewsAction) *aNewsAction = m_newsAction;
   return NS_OK;
 }
@@ -246,33 +241,42 @@ NS_IMETHODIMP nsNntpUrl::SetNewsAction(nsNewsAction aNewsAction) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetGroup(nsACString &group) {
+NS_IMETHODIMP nsNntpUrl::GetGroup(nsACString& group) {
   group = m_group;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetMessageID(nsACString &messageID) {
+NS_IMETHODIMP nsNntpUrl::GetMessageID(nsACString& messageID) {
   messageID = m_messageID;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetKey(nsMsgKey *key) {
+NS_IMETHODIMP nsNntpUrl::GetKey(nsMsgKey* key) {
   NS_ENSURE_ARG_POINTER(key);
   *key = m_key;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetNormalizedSpec(nsACString &aPrincipalSpec) {
+NS_IMETHODIMP nsNntpUrl::GetCharset(nsACString& charset) {
+  nsCOMPtr<nsIMsgIncomingServer> server;
+  nsresult rv = GetServer(getter_AddRefs(server));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsINntpIncomingServer> nserver(do_QueryInterface(server));
+  NS_ENSURE_TRUE(nserver, NS_ERROR_NULL_POINTER);
+  return nserver->GetCharset(charset);
+}
+
+NS_IMETHODIMP nsNntpUrl::GetNormalizedSpec(nsACString& aPrincipalSpec) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsNntpUrl::SetUri(const char *aURI) {
+NS_IMETHODIMP nsNntpUrl::SetUri(const nsACString& aURI) {
   mURI = aURI;
   return NS_OK;
 }
 
 // from nsIMsgMessageUrl
-NS_IMETHODIMP nsNntpUrl::GetUri(char **aURI) {
+NS_IMETHODIMP nsNntpUrl::GetUri(nsACString& aURI) {
   nsresult rv = NS_OK;
 
   // if we have been given a uri to associate with this url, then use it
@@ -284,20 +288,19 @@ NS_IMETHODIMP nsNntpUrl::GetUri(char **aURI) {
     mURI = spec;
   }
 
-  *aURI = ToNewCString(mURI);
-  if (!*aURI) return NS_ERROR_OUT_OF_MEMORY;
+  aURI = mURI;
   return rv;
 }
 
 NS_IMPL_GETSET(nsNntpUrl, AddDummyEnvelope, bool, m_addDummyEnvelope)
 NS_IMPL_GETSET(nsNntpUrl, CanonicalLineEnding, bool, m_canonicalLineEnding)
 
-NS_IMETHODIMP nsNntpUrl::SetMessageFile(nsIFile *aFile) {
+NS_IMETHODIMP nsNntpUrl::SetMessageFile(nsIFile* aFile) {
   m_messageFile = aFile;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetMessageFile(nsIFile **aFile) {
+NS_IMETHODIMP nsNntpUrl::GetMessageFile(nsIFile** aFile) {
   if (aFile) NS_IF_ADDREF(*aFile = m_messageFile);
   return NS_OK;
 }
@@ -306,31 +309,11 @@ NS_IMETHODIMP nsNntpUrl::GetMessageFile(nsIFile **aFile) {
 // End nsINntpUrl specific support
 ////////////////////////////////////////////////////////////////////////////////
 
-nsresult nsNntpUrl::SetMessageToPost(nsINNTPNewsgroupPost *post) {
-  m_newsgroupPost = post;
-  if (post) SetNewsAction(nsINntpUrl::ActionPostArticle);
-  return NS_OK;
-}
-
-nsresult nsNntpUrl::GetMessageToPost(nsINNTPNewsgroupPost **aPost) {
-  NS_ENSURE_ARG_POINTER(aPost);
-  NS_IF_ADDREF(*aPost = m_newsgroupPost);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsNntpUrl::SetMessageHeader(nsIMsgDBHdr *aMsgHdr) {
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsNntpUrl::GetMessageHeader(nsIMsgDBHdr **aMsgHdr) {
+NS_IMETHODIMP nsNntpUrl::GetMessageHeader(nsIMsgDBHdr** aMsgHdr) {
   nsresult rv;
 
-  nsCOMPtr<nsINntpService> nntpService =
-      do_GetService(NS_NNTPSERVICE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsCOMPtr<nsIMsgMessageService> msgService =
-      do_QueryInterface(nntpService, &rv);
+      do_GetService("@mozilla.org/messenger/messageservice;1?type=news", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoCString spec(mOriginalSpec);
@@ -342,10 +325,10 @@ NS_IMETHODIMP nsNntpUrl::GetMessageHeader(nsIMsgDBHdr **aMsgHdr) {
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  return msgService->MessageURIToMsgHdr(spec.get(), aMsgHdr);
+  return msgService->MessageURIToMsgHdr(spec, aMsgHdr);
 }
 
-NS_IMETHODIMP nsNntpUrl::IsUrlType(uint32_t type, bool *isType) {
+NS_IMETHODIMP nsNntpUrl::IsUrlType(uint32_t type, bool* isType) {
   NS_ENSURE_ARG(isType);
 
   switch (type) {
@@ -360,21 +343,19 @@ NS_IMETHODIMP nsNntpUrl::IsUrlType(uint32_t type, bool *isType) {
 }
 
 NS_IMETHODIMP
-nsNntpUrl::GetOriginalSpec(char **aSpec) {
-  NS_ENSURE_ARG_POINTER(aSpec);
-  *aSpec = ToNewCString(mOriginalSpec);
-  if (!*aSpec) return NS_ERROR_OUT_OF_MEMORY;
+nsNntpUrl::GetOriginalSpec(nsACString& aSpec) {
+  aSpec = mOriginalSpec;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpUrl::SetOriginalSpec(const char *aSpec) {
+nsNntpUrl::SetOriginalSpec(const nsACString& aSpec) {
   mOriginalSpec = aSpec;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpUrl::GetServer(nsIMsgIncomingServer **aServer) {
+nsNntpUrl::GetServer(nsIMsgIncomingServer** aServer) {
   NS_ENSURE_ARG_POINTER(aServer);
 
   nsresult rv;
@@ -390,42 +371,16 @@ nsNntpUrl::GetServer(nsIMsgIncomingServer **aServer) {
     return NS_OK;
   }
 
-  // Looking up the server...
-  // news-message is used purely internally, so it can never refer to the real
-  // attribute. nntp is never used internally, so it probably refers to the real
-  // one. news is used both internally and externally, so it could refer to
-  // either one. We'll assume it's an internal one first, though.
-  bool isNews = scheme.EqualsLiteral("news") || scheme.EqualsLiteral("snews");
-  bool isNntp = scheme.EqualsLiteral("nntp") || scheme.EqualsLiteral("nntps");
-
-  bool tryReal = isNntp;
-
   nsCOMPtr<nsIMsgAccountManager> accountManager =
-      do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+      do_GetService("@mozilla.org/messenger/account-manager;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // Ignoring return results: it is perfectly acceptable for the server to not
-  // exist, but FindServer (and not FindRealServer) throws NS_ERROR_UNEXPECTED
-  // in this case.
   *aServer = nullptr;
-  if (tryReal)
-    accountManager->FindRealServer(user, host, NS_LITERAL_CSTRING("nntp"), 0,
-                                   aServer);
-  else
-    accountManager->FindServer(user, host, NS_LITERAL_CSTRING("nntp"), aServer);
-  if (!*aServer && (isNews || isNntp)) {
-    // Didn't find it, try the other option
-    if (tryReal)
-      accountManager->FindServer(user, host, NS_LITERAL_CSTRING("nntp"),
-                                 aServer);
-    else
-      accountManager->FindRealServer(user, host, NS_LITERAL_CSTRING("nntp"), 0,
-                                     aServer);
-  }
+  accountManager->FindServer(user, host, "nntp"_ns, 0, aServer);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::GetFolder(nsIMsgFolder **msgFolder) {
+NS_IMETHODIMP nsNntpUrl::GetFolder(nsIMsgFolder** msgFolder) {
   NS_ENSURE_ARG_POINTER(msgFolder);
 
   nsresult rv;
@@ -458,46 +413,20 @@ NS_IMETHODIMP nsNntpUrl::GetFolder(nsIMsgFolder **msgFolder) {
   NS_ENSURE_SUCCESS(rv, rv);
 
   return newsFolder->QueryInterface(NS_GET_IID(nsIMsgFolder),
-                                    (void **)msgFolder);
+                                    (void**)msgFolder);
 }
 
-NS_IMETHODIMP
-nsNntpUrl::GetFolderCharset(char **aCharacterSet) {
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetFolder(getter_AddRefs(folder));
-  // don't assert here.  this can happen if there is no message folder
-  // like when we display a news://host/message-id url
-  if (NS_FAILED(rv) || !folder) return rv;
-  nsCString tmpStr;
-  rv = folder->GetCharset(tmpStr);
-  *aCharacterSet = ToNewCString(tmpStr);
-  return rv;
-}
-
-NS_IMETHODIMP nsNntpUrl::GetFolderCharsetOverride(bool *aCharacterSetOverride) {
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetFolder(getter_AddRefs(folder));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(folder, NS_ERROR_FAILURE);
-  rv = folder->GetCharsetOverride(aCharacterSetOverride);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return rv;
-}
-
-NS_IMETHODIMP nsNntpUrl::GetCharsetOverRide(char **aCharacterSet) {
-  if (!mCharsetOverride.IsEmpty())
-    *aCharacterSet = ToNewCString(mCharsetOverride);
-  else
-    *aCharacterSet = nullptr;
+NS_IMETHODIMP nsNntpUrl::GetAutodetectCharset(bool* aAutodetectCharset) {
+  *aAutodetectCharset = mAutodetectCharset;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpUrl::SetCharsetOverRide(const char *aCharacterSet) {
-  mCharsetOverride = aCharacterSet;
+NS_IMETHODIMP nsNntpUrl::SetAutodetectCharset(bool aAutodetectCharset) {
+  mAutodetectCharset = aAutodetectCharset;
   return NS_OK;
 }
 
-nsresult nsNntpUrl::Clone(nsIURI **_retval) {
+nsresult nsNntpUrl::Clone(nsIURI** _retval) {
   nsresult rv;
   rv = nsMsgMailNewsUrl::Clone(_retval);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -505,5 +434,28 @@ nsresult nsNntpUrl::Clone(nsIURI **_retval) {
   nsCOMPtr<nsIMsgMessageUrl> newsurl = do_QueryInterface(*_retval, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return newsurl->SetUri(mURI.get());
+  return newsurl->SetUri(mURI);
+}
+
+nsresult nsNntpUrl::NewURI(const nsACString& aSpec, nsIURI* aBaseURI,
+                           nsIURI** _retval) {
+  nsresult rv;
+
+  nsCOMPtr<nsIMsgMailNewsUrl> nntpUri =
+      do_CreateInstance("@mozilla.org/messenger/nntpurl;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (aBaseURI) {
+    nsAutoCString newSpec;
+    aBaseURI->Resolve(aSpec, newSpec);
+    rv = nntpUri->SetSpecInternal(newSpec);
+    // XXX Consider: rv = NS_MutateURI(new
+    // nsNntpUrl::Mutator()).SetSpec(newSpec).Finalize(nntpUri);
+  } else {
+    rv = nntpUri->SetSpecInternal(aSpec);
+  }
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nntpUri.forget(_retval);
+  return NS_OK;
 }
