@@ -1,9 +1,9 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from ../../../mail/components/addrbook/content/abSearchDialog.js */
+// abSearchDialog.js
+/* globals GetScopeForDirectoryURI */
 
 var gTotalSearchTerms = 0;
 var gSearchTermList;
@@ -115,19 +115,18 @@ searchTermContainer.prototype = {
 
   save() {
     var searchTerm = this.searchTerm;
-    var nsMsgSearchAttrib = Ci.nsMsgSearchAttrib;
 
     if (isNaN(this.searchattribute.value)) {
       // is this a custom term?
-      searchTerm.attrib = nsMsgSearchAttrib.Custom;
+      searchTerm.attrib = Ci.nsMsgSearchAttrib.Custom;
       searchTerm.customId = this.searchattribute.value;
     } else {
       searchTerm.attrib = this.searchattribute.value;
     }
 
     if (
-      this.searchattribute.value > nsMsgSearchAttrib.OtherHeader &&
-      this.searchattribute.value < nsMsgSearchAttrib.kNumMsgSearchAttributes
+      this.searchattribute.value > Ci.nsMsgSearchAttrib.OtherHeader &&
+      this.searchattribute.value < Ci.nsMsgSearchAttrib.kNumMsgSearchAttributes
     ) {
       searchTerm.arbitraryHeader = this.searchattribute.label;
     }
@@ -153,9 +152,11 @@ function initializeSearchWidgets() {
   gSearchTermList = document.getElementById("searchTermList");
 
   // initialize some strings
-  var bundle = document.getElementById("bundle_search");
-  gMoreButtonTooltipText = bundle.getString("moreButtonTooltipText");
-  gLessButtonTooltipText = bundle.getString("lessButtonTooltipText");
+  var bundle = Services.strings.createBundle(
+    "chrome://messenger/locale/search.properties"
+  );
+  gMoreButtonTooltipText = bundle.GetStringFromName("moreButtonTooltipText");
+  gLessButtonTooltipText = bundle.GetStringFromName("lessButtonTooltipText");
 }
 
 function initializeBooleanWidgets() {
@@ -190,7 +191,7 @@ function initializeBooleanWidgets() {
 
 function initializeSearchRows(scope, searchTerms) {
   for (let i = 0; i < searchTerms.length; i++) {
-    let searchTerm = searchTerms[i];
+    const searchTerm = searchTerms[i];
     createSearchRow(i, scope, searchTerm, false);
     gTotalSearchTerms++;
   }
@@ -201,14 +202,13 @@ function initializeSearchRows(scope, searchTerms) {
 /**
  * Enables/disables all the visible elements inside the search terms listbox.
  *
- * @param matchAllValue boolean value from the first search term
+ * @param {boolean} matchAllValue - value from the first search term
  */
 function updateSearchTermsListbox(matchAllValue) {
   var searchTerms = document.getElementById("searchTermList");
   searchTerms.setAttribute("disabled", matchAllValue);
-  var searchAttributeList = searchTerms.getElementsByTagName(
-    "search-attribute"
-  );
+  var searchAttributeList =
+    searchTerms.getElementsByTagName("search-attribute");
   var searchOperatorList = searchTerms.getElementsByTagName("search-operator");
   var searchValueList = searchTerms.getElementsByTagName("search-value");
   for (let i = 0; i < searchAttributeList.length; i++) {
@@ -330,11 +330,12 @@ function booleanChanged(event) {
 /**
  * Create a new search row with all the needed elements.
  *
- * @param index       index of the position in the menulist where to add the row
- * @param scope       a nsMsgSearchScope constant indicating scope of this search rule
- * @param searchTerm  nsIMsgSearchTerm object to hold the search term
- * @param aUserAdded  boolean indicating if the row addition was initiated by the user
- *                    (e.g. via the '+' button)
+ * @param {integer} index - Index of the position in the menulist where to add the row.
+ * @param {nsMsgSearchScope} scope - A nsMsgSearchScope constant indicating
+ *   scope of this search rule.
+ * @param {nsIMsgSearchTerm} searchTerm  - The search term.
+ * @param {boolean} aUserAdded - Boolean indicating if the row addition was
+ *   initiated by the user (e.g. via the '+' button).
  */
 function createSearchRow(index, scope, searchTerm, aUserAdded) {
   var searchAttr = document.createXULElement("search-attribute");
@@ -442,21 +443,21 @@ function initializeTermFromIndex(index) {
 /**
  * Creates a <richlistitem> using the array children as the children
  * of each listcell.
- * @param aChildren  An array of XUL elements to put into the listitem.
- *                   Each array member is put into a separate listcell.
- *                   If the member itself is an array of elements,
- *                   all of them are put into the same listcell.
+ *
+ * @param {Element[]} aChildren - An array of XUL elements to put into the
+ *   listitem. Each array member is put into a separate listcell. If the member
+ *   itself is an array of elements, all of them are put into the same listcell.
  */
 function constructRow(aChildren) {
-  let cols = gSearchTermList.firstElementChild.children; // treecol elements
-  let listitem = document.createXULElement("richlistitem");
+  const cols = gSearchTermList.firstElementChild.children; // treecol elements
+  const listitem = document.createXULElement("richlistitem");
   listitem.setAttribute("allowevents", "true");
   for (let i = 0; i < aChildren.length; i++) {
-    let listcell = document.createXULElement("hbox");
-    if (cols[i].hasAttribute("flex")) {
-      listcell.setAttribute("flex", cols[i].getAttribute("flex"));
+    const listcell = document.createXULElement("hbox");
+    if (cols[i].hasAttribute("style")) {
+      listcell.setAttribute("style", cols[i].getAttribute("style"));
     }
-    let child = aChildren[i];
+    const child = aChildren[i];
 
     if (child instanceof Array) {
       for (let j = 0; j < child.length; j++) {
@@ -471,6 +472,9 @@ function constructRow(aChildren) {
   return listitem;
 }
 
+/**
+ * @param {integer} index - Index of row to remove.
+ */
 function removeSearchRow(index) {
   var searchTermObj = gSearchTerms[index].obj;
   if (!searchTermObj) {
@@ -513,8 +517,9 @@ function removeSearchRow(index) {
 
 /**
  * Save the search terms from the UI back to the actual search terms.
+ *
  * @param {nsIMsgSearchTerm[]} searchTerms - Array of terms
- * @param {Object} termOwner - Object which can contain and create the terms
+ * @param {object} termOwner - Object which can contain and create the terms
  *   e.g. a nsIMsgSearchSession (will be unnecessary if we just make terms
  *   creatable via XPCOM).
  * @returns {nsIMsgSearchTerm[]} The filtered searchTerms.
@@ -550,7 +555,7 @@ function saveSearchTerms(searchTerms, termOwner) {
   return searchTerms;
 }
 
-function onReset(event) {
+function onReset() {
   while (gTotalSearchTerms > 0) {
     removeSearchRow(--gTotalSearchTerms);
   }
