@@ -7,12 +7,11 @@
 #define _nsMsgFilter_H_
 
 #include "nscore.h"
+#include "nsCOMPtr.h"
 #include "nsISupports.h"
 #include "nsIMsgFilter.h"
 #include "nsIMsgSearchScopeTerm.h"
 #include "nsMsgSearchBoolExpression.h"
-#include "nsIMutableArray.h"
-#include "DateTimeFormat.h"
 #include "nsIMsgFilterCustomAction.h"
 
 class nsMsgRuleAction : public nsIMsgRuleAction {
@@ -29,7 +28,6 @@ class nsMsgRuleAction : public nsIMsgRuleAction {
   nsMsgRuleActionType m_type;
   // this used to be a union - why bother?
   nsMsgPriorityValue m_priority; /* priority to set rule to */
-  nsMsgLabelValue m_label;       /* label to set rule to */
   nsCString m_folderUri;
   int32_t m_junkScore; /* junk score (or arbitrary int value?) */
   // arbitrary string value. Currently, email address to forward to
