@@ -4,14 +4,17 @@
 
 const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { ExtensionSupport } = ChromeUtils.importESModule(
+  "resource:///modules/ExtensionSupport.sys.mjs"
+);
+
 var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-var { migrateMailnews } =
-  ChromeUtils.import("resource:///modules/mailnewsMigrator.js");
-var { ExtensionSupport } = 
-  ChromeUtils.import("resource:///modules/ExtensionSupport.jsm");
-var { LightweightThemeConsumer } =
-  ChromeUtils.import("resource://gre/modules/LightweightThemeConsumer.jsm");
+var { migrateMailnews } = ChromeUtils.import(
+  "resource:///modules/mailnewsMigrator.js"
+);
+var { LightweightThemeConsumer } = ChromeUtils.import(
+  "resource://gre/modules/LightweightThemeConsumer.jsm"
+);
 
 XPCOMUtils.defineLazyModuleGetters(this, {
   AddonManager: "resource://gre/modules/AddonManager.jsm",
@@ -23,6 +26,7 @@ XPCOMUtils.defineLazyModuleGetters(this, {
   PlacesBackups: "resource://gre/modules/PlacesBackups.jsm",
   AsyncShutdown: "resource://gre/modules/AsyncShutdown.jsm",
   AutoCompletePopup: "resource://gre/modules/AutoCompletePopup.jsm",
+  DateTimePickerHelper: "resource://gre/modules/DateTimePickerHelper.jsm",
   BookmarkHTMLUtils: "resource://gre/modules/BookmarkHTMLUtils.jsm",
   BookmarkJSONUtils: "resource://gre/modules/BookmarkJSONUtils.jsm",
   RecentWindow: "resource:///modules/RecentWindow.jsm",
@@ -100,7 +104,7 @@ function SuiteGlue() {
                                      "nsIIdleService");
 
   this._init();
-  extensionDefaults(); // extensionSupport.jsm
+  extensionDefaults(); // ExtensionSupport.sys.mjs
 }
 
 SuiteGlue.prototype = {
@@ -411,7 +415,7 @@ SuiteGlue.prototype = {
    * level.
    */
   _migrateUI() {
-    const UI_VERSION = 8;
+    const UI_VERSION = 10;
 
     // If the pref is not set this is a new or pre SeaMonkey 2.49 profile.
     // We can't tell so we just run migration with version 0.
@@ -566,6 +570,57 @@ SuiteGlue.prototype = {
       Services.prefs.clearUserPref("privacy.sanitize.sanitizeOnShutdown");
     }
 
+    // Migrate mail tab options.
+    if (currentUIVersion < 9) {
+      const tabPrefs = [ "autoHide", "opentabfor.doubleclick",
+                         "opentabfor.middleclick" ];
+      for (let pref of tabPrefs) {
+        try {
+          let prefBT = "browser.tabs." + pref;
+
+          // Copy user value otherwise use default.
+          if (Services.prefs.prefHasUserValue(prefBT)) {
+            let prefMT = "mail.tabs." + pref;
+
+            // If it has a value this should never fail.
+            let valueBT = Services.prefs.getBoolPref(prefBT);
+            Services.prefs.setBoolPref(prefMT, valueBT);
+          }
+        } catch (ex) {
+          // Better safe than sorry.
+          Cu.reportError(ex);
+        }
+      }
+
+      // We might bring this back later but currently set to default.
+      Services.prefs.clearUserPref("browser.tabs.opentabfor.doubleclick");
+    }
+
+    // Migrate the old requested locales prefs to use the new model
+    if (currentUIVersion < 10) {
+      const SELECTED_LOCALE_PREF = "general.useragent.locale";
+      const MATCHOS_LOCALE_PREF = "intl.locale.matchOS";
+
+      if (Services.prefs.prefHasUserValue(MATCHOS_LOCALE_PREF) ||
+          Services.prefs.prefHasUserValue(SELECTED_LOCALE_PREF)) {
+        if (Services.prefs.getBoolPref(MATCHOS_LOCALE_PREF, false)) {
+          Services.locale.setRequestedLocales([]);
+        } else {
+          let locale = Services.prefs.getComplexValue(SELECTED_LOCALE_PREF,
+            Ci.nsIPrefLocalizedString);
+          if (locale) {
+            try {
+              Services.locale.setRequestedLocales([locale.data]);
+            } catch (e) {
+              /* Don't panic if the value is not a valid locale code. */
+            }
+          }
+        }
+        Services.prefs.clearUserPref(SELECTED_LOCALE_PREF);
+        Services.prefs.clearUserPref(MATCHOS_LOCALE_PREF);
+      }
+    }
+
     // Update the migration version.
     Services.prefs.setIntPref("suite.migration.version", UI_VERSION);
   },
@@ -702,6 +757,7 @@ SuiteGlue.prototype = {
   // First mail or browser window loaded.
   _onFirstWindowLoaded: function(aWindow) {
     AutoCompletePopup.init();
+    DateTimePickerHelper.init();
 
     if ("@mozilla.org/windows-taskbar;1" in Cc &&
         Cc["@mozilla.org/windows-taskbar;1"]
@@ -738,6 +794,7 @@ SuiteGlue.prototype = {
       this._setPrefToSaveSession();
     }
     AutoCompletePopup.uninit();
+    DateTimePickerHelper.uninit();
   },
 
   _promptForMasterPassword: function()
@@ -1276,10 +1333,8 @@ SuiteGlue.prototype = {
     // Make sure chrome debugging is enabled, no sense in starting otherwise.
     DebuggerServer.allowChromeProcess = true;
 
-    if (!DebuggerServer.initialized) {
-      DebuggerServer.init();
-      DebuggerServer.addBrowserActors();
-    }
+    DebuggerServer.init();
+    DebuggerServer.registerAllActors();
     try {
       let listener = DebuggerServer.createListener();
       listener.portOrPath = port;
@@ -1499,8 +1554,7 @@ SuiteGlue.prototype = {
         var args = Cc["@mozilla.org/supports-string;1"]
                      .createInstance(Ci.nsISupportsString);
         args.data = url;
-        var chromeURL = Services.prefs.getCharPref("browser.chromeURL");
-        Services.ww.openWindow(null, chromeURL, "_blank", "chrome,all,dialog=no", args);
+        Services.ww.openWindow(null, AppConstants.BROWSER_CHROME_URL, "_blank", "chrome,all,dialog=no", args);
       }
     } catch (e) {
       Cu.reportError("Error displaying tab received by Sync: " + e);
@@ -1510,10 +1564,10 @@ SuiteGlue.prototype = {
   // for XPCOM
   classID: Components.ID("{bbbbe845-5a1b-40ee-813c-f84b8faaa07c}"),
 
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver,
-                                         Ci.nsIWebProgressListener,
-                                         Ci.nsISupportsWeakReference,
-                                         Ci.nsISuiteGlue])
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIObserver,
+                                          Ci.nsIWebProgressListener,
+                                          Ci.nsISupportsWeakReference,
+                                          Ci.nsISuiteGlue])
 
 }
 
@@ -1556,9 +1610,7 @@ var ContentPermissionIntegration = {
         return new PermissionUI.DesktopNotificationPermissionPrompt(request);
       }
       case "persistent-storage": {
-        if (Services.prefs.getBoolPref("browser.storageManager.enabled")) {
-          return new PermissionUI.PersistentStoragePermissionPrompt(request);
-        }
+        return new PermissionUI.PersistentStoragePermissionPrompt(request);
       }
     }
     return undefined;
@@ -1570,7 +1622,7 @@ function ContentPermissionPrompt() {}
 ContentPermissionPrompt.prototype = {
   classID: Components.ID("{9d4c845d-3f09-402a-b66d-50f291d7d50f}"),
 
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPermissionPrompt]),
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIContentPermissionPrompt]),
 
   /**
    * This implementation of nsIContentPermissionPrompt.prompt ensures
