@@ -5,7 +5,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
 ChromeUtils.defineModuleGetter(this, "PlacesUtils",
                                "resource://gre/modules/PlacesUtils.jsm");
 ChromeUtils.defineModuleGetter(this, "NetUtil",
@@ -271,7 +270,7 @@ AutoCompleteStatementCallbackWrapper.prototype = {
   //////////////////////////////////////////////////////////////////////////////
   //// nsISupports
 
-  QueryInterface: XPCOMUtils.generateQI([
+  QueryInterface: ChromeUtils.generateQI([
     Ci.mozIStorageStatementCallback,
   ])
 };
@@ -453,17 +452,12 @@ function nsPlacesAutoComplete()
   //// Initialization
 
   // load preferences
-  this._prefs = Cc["@mozilla.org/preferences-service;1"].
-                getService(Ci.nsIPrefService).
-                getBranch(kBrowserUrlbarBranch);
+  this._prefs = Services.prefs.getBranch(kBrowserUrlbarBranch);
   this._syncEnabledPref();
   this._loadPrefs(true);
 
   // register observers
-  this._os = Cc["@mozilla.org/observer-service;1"].
-              getService(Ci.nsIObserverService);
-  this._os.addObserver(this, kTopicShutdown);
-
+  Services.obs.addObserver(this, kTopicShutdown);
 }
 
 nsPlacesAutoComplete.prototype = {
@@ -655,7 +649,7 @@ nsPlacesAutoComplete.prototype = {
   observe: function PAC_observe(aSubject, aTopic, aData)
   {
     if (aTopic == kTopicShutdown) {
-      this._os.removeObserver(this, kTopicShutdown);
+      Services.obs.removeObserver(this, kTopicShutdown);
 
       // Remove our preference observer.
       this._prefs.removeObserver("", this);
@@ -1314,7 +1308,7 @@ nsPlacesAutoComplete.prototype = {
 
   _xpcom_factory: XPCOMUtils.generateSingletonFactory(nsPlacesAutoComplete),
 
-  QueryInterface: XPCOMUtils.generateQI([
+  QueryInterface: ChromeUtils.generateQI([
     Ci.nsIAutoCompleteSearch,
     Ci.nsIAutoCompleteSimpleResultListener,
     Ci.mozIPlacesAutoComplete,
