@@ -1,14 +1,12 @@
-/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-
 /*
  * Command-specific code. This stuff should be called by the widgets
  */
  
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 const { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.js");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
@@ -871,7 +869,7 @@ function  CreateVirtualFolder(newName, parentFolder, searchFolderURIs, searchTer
       dbFolderInfo.setBooleanProperty("searchOnline", searchOnline);
       vfdb.summaryValid = true;
       vfdb.Close(true);
-      parentFolder.NotifyItemAdded(newFolder);
+      parentFolder.notifyFolderAdded(newFolder);
       MailServices.accounts.saveVirtualFolders();
     }
     catch(e)
