@@ -1,9 +1,10 @@
-/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* This is where functions related to the 3 pane window are kept */
+const { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.js");
 const {msgDBCacheManager} = ChromeUtils.import("resource:///modules/msgDBCacheManager.js");
 const {PeriodicFilterManager} = ChromeUtils.import("resource:///modules/PeriodicFilterManager.jsm");
 
@@ -12,7 +13,6 @@ const nsMsgKey_None = 0xFFFFFFFF;
 const nsMsgViewIndex_None = 0xFFFFFFFF;
 const kMailCheckOncePrefName = "mail.startup.enabledMailCheckOnce";
 
-var gFolderTree;
 var gSearchInput;
 
 var gUnreadCount = null;
@@ -76,7 +76,7 @@ function ScrollToMessageAfterFolderLoad(folder)
 {
   var scrolled = Services.prefs.getBoolPref("mailnews.scroll_to_new_message") &&
       ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
-  if (!scrolled && folder && Services.prefs.getBoolPref("mailnews.remember_selected_message"))
+  if (!scrolled && folder)
   {
     // If we failed to scroll to a new message,
     // reselect the last selected message
@@ -113,15 +113,17 @@ function ScrollToMessageAfterFolderLoad(folder)
 // the folderListener object
 var folderListener =
 {
-  OnItemAdded:   function(parentItem, item) {},
-  OnItemRemoved: function(parentItem, item) {},
+  onFolderAdded: function(parentFolder, child) {},
+  onMessageAdded: function(parentFolder, msg) {},
+  onFolderRemoved: function(parentFolder, child) {},
+  onMessageRemoved: function(parentFolder, msg) {},
 
-  OnItemPropertyChanged:        function(item, property, oldValue, newValue) {},
-  OnItemBoolPropertyChanged:    function(item, property, oldValue, newValue) {},
-  OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue) {},
-  OnItemPropertyFlagChanged:    function(item, property, oldFlag,  newFlag)  {},
+  onFolderPropertyChanged:        function(item, property, oldValue, newValue) {},
+  onFolderBoolPropertyChanged:    function(item, property, oldValue, newValue) {},
+  onFolderUnicharPropertyChanged: function(item, property, oldValue, newValue) {},
+  onFolderPropertyFlagChanged:    function(item, property, oldFlag,  newFlag)  {},
 
-  OnItemIntPropertyChanged: function(item, property, oldValue, newValue)
+  onFolderIntPropertyChanged: function(item, property, oldValue, newValue)
   {
     // handle the currently visible folder
     if (item == gMsgFolderSelected)
@@ -129,8 +131,6 @@ var folderListener =
       if (property == "TotalMessages" || property == "TotalUnreadMessages")
       {
         UpdateStatusMessageCounts(gMsgFolderSelected);
-        item = item.QueryInterface(Ci.nsIRDFResource);
-        UpdateLocationBar(item);
       }
     }
 
@@ -153,10 +153,9 @@ var folderListener =
     }
   },
 
-    OnItemEvent: function(folder, event) {
+    onFolderEvent: function(folder, event) {
       if (event == "FolderLoaded") {
         if (folder) {
-          const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
           var scrolled = false;
           var msgFolder = folder.QueryInterface(Ci.nsIMsgFolder);
           var uri = folder.URI;
@@ -210,15 +209,19 @@ var folderListener =
             gCurrentLoadingFolderURI = "";
             // Scroll to message for virtual folders is done in
             // gSearchNotificationListener.OnSearchDone (see searchBar.js).
-            if (!scrolled && !(gMsgFolderSelected.flags & nsMsgFolderFlags.Virtual))
+            if (!scrolled && gMsgFolderSelected &&
+                !(gMsgFolderSelected.flags & Ci.nsMsgFolderFlags.Virtual))
               ScrollToMessageAfterFolderLoad(msgFolder);
             SetBusyCursor(window, false);
           }
           // Folder loading is over,
           // now issue quick search if there is an email address.
-          viewDebug("in folder loaded gVirtualFolderTerms = " + gVirtualFolderTerms + "\n");
-          viewDebug("in folder loaded gMsgFolderSelected = " +
-                    gMsgFolderSelected && gMsgFolderSelected.URI + "\n");
+          if (gVirtualFolderTerms)
+            viewDebug("in folder loaded gVirtualFolderTerms = " +
+                      gVirtualFolderTerms + "\n");
+          if (gMsgFolderSelected)
+            viewDebug("in folder loaded gMsgFolderSelected = " +
+                      gMsgFolderSelected.URI + "\n");
           if (rerootingFolder)
           {
             if (gSearchEmailAddress)
@@ -233,7 +236,8 @@ var folderListener =
               gDBView.viewFolder = gMsgFolderSelected;
               ViewChangeByFolder(gMsgFolderSelected);
             }
-            else if (gMsgFolderSelected.flags & nsMsgFolderFlags.Virtual)
+            else if (gMsgFolderSelected &&
+                     gMsgFolderSelected.flags & Ci.nsMsgFolderFlags.Virtual)
             {
               viewDebug("selected folder is virtual\n");
               gDefaultSearchViewTerms = null;
@@ -272,11 +276,11 @@ var folderListener =
               if (msgHdr)
               {
                 var hdrs = hdrParser.headers;
-                if (hdrs && hdrs.indexOf("X-attachment-size:") > 0) {
+                if (hdrs && hdrs.includes("X-attachment-size:")) {
                   msgHdr.OrFlags(Ci.nsMsgMessageFlags
                                    .Attachment);
                 }
-                if (hdrs && hdrs.indexOf("X-image-size:") > 0) {
+                if (hdrs && hdrs.includes("X-image-size:")) {
                   msgHdr.setStringProperty("imageSize", "1");
                 }
               }
@@ -300,7 +304,7 @@ var folderListener =
       else if (event == "RenameCompleted") {
         // Clear this so we don't try to clear its new messages.
         gMsgFolderSelected = null;
-        SelectFolder(folder.URI);
+        gFolderTreeView.selectFolder(folder);
       }
       else if (event == "JunkStatusChanged") {
         HandleJunkStatusChanged(folder);
@@ -308,67 +312,6 @@ var folderListener =
     }
 }
 
-var folderObserver = {
-    canDrop: function(index, orientation)
-    {
-        return CanDropOnFolderTree(index, orientation);
-    },
-
-    onDrop: function(row, orientation)
-    {
-        DropOnFolderTree(row, orientation);
-    },
-
-    onToggleOpenState: function(index)
-    {
-      var folderTree = GetFolderTree();
-
-      // Nothing to do when collapsing an item.
-      if (folderTree.view.isContainerOpen(index))
-        return;
-
-      var folderResource = GetFolderResource(folderTree, index);
-
-      if (folderTree.view.getLevel(index) == 0)
-      {
-        // (Imap/Nntp/Pop) Account item.
-
-        folderResource.QueryInterface(Ci.nsIMsgFolder)
-                      .server.performExpand(msgWindow);
-      }
-      else if (folderResource instanceof Ci.nsIMsgImapMailFolder)
-      {
-        // Imap message folder item.
-
-        folderResource.performExpand(msgWindow);
-      }
-    },
-
-    onCycleHeader: function(colID, elt)
-    {
-    },
-
-    onCycleCell: function(row, colID)
-    {
-    },
-
-    onSelectionChanged: function()
-    {
-    },
-
-    onPerformAction: function(action)
-    {
-    },
-
-    onPerformActionOnRow: function(action, row)
-    {
-    },
-
-    onPerformActionOnCell: function(action, row, colID)
-    {
-    }
-}
-
 function HandleDeleteOrMoveMsgFailed(folder)
 {
   gDBView.onDeleteCompleted(false);
@@ -558,7 +501,7 @@ function IsCurrentLoadedFolder(aFolder)
   {
     return currentLoadedFolder.msgDatabase.dBFolderInfo
                               .getCharProperty("searchFolderUri").split("|")
-                              .indexOf(msgFolderUri) != -1;
+                              .includes(msgFolderUri);
   }
 
   // Is aFolder the currently loaded folder?
@@ -575,7 +518,7 @@ function ServerContainsFolder(server, folder)
 
 function SelectServer(server)
 {
-  SelectFolder(server.rootFolder.URI);
+  gFolderTreeView.selectFolder(server.rootFolder);
 }
 
 // we have this incoming server listener in case we need to
@@ -649,7 +592,7 @@ function UpdateMailPaneConfig() {
   }
 }
 
-const MailPrefObserver = {
+var MailPrefObserver = {
   observe: function observe(subject, topic, prefName) {
     if (topic == "nsPref:changed") {
       if (prefName == "mail.pane_config.dynamic") {
@@ -711,7 +654,6 @@ function OnLoadMessenger()
   InitMsgWindow();
   messenger.setWindow(window, msgWindow);
 
-  InitializeDataSources();
   InitPanes();
 
   MigrateJunkMailSettings();
@@ -832,32 +774,8 @@ function MailWindowIsClosing(aCancelQuit, aTopic, aData)
       aCancelQuit.data)
     return false;
 
-  let reallyClose = true;
-  let numtabs = GetTabMail().tabInfo.length;
-  if (numtabs > 1)
-  {
-    let shouldPrompt = Services.prefs.getBoolPref("browser.tabs.warnOnClose");
-    if (shouldPrompt)
-    {
-      // default to true: if it were false, we wouldn't get this far
-      let warnOnClose = {value: true};
-      let buttonPressed = Services.prompt.confirmEx(
-        window,
-        gMessengerBundle.getString('tabs.closeWarningTitle'),
-        gMessengerBundle.getFormattedString("tabs.closeWarning", [numtabs], 1),
-        (Services.prompt.BUTTON_TITLE_IS_STRING * Services.prompt.BUTTON_POS_0) +
-        (Services.prompt.BUTTON_TITLE_CANCEL * Services.prompt.BUTTON_POS_1),
-        gMessengerBundle.getString('tabs.closeButton'),
-        null,
-        null,
-        gMessengerBundle.getString('tabs.closeWarningPromptMe'),
-        warnOnClose);
-      reallyClose = (buttonPressed == 0);
-      // don't set the pref unless OK was pressed and it's false
-      if (reallyClose && !warnOnClose.value)
-        Services.prefs.setBoolPref("browser.tabs.warnOnClose", false);
-    }
-  }
+  let tabmail = GetTabMail();
+  let reallyClose = tabmail.warnAboutClosingTabs(tabmail.closingTabsEnum.ALL);
 
   if (!reallyClose && aTopic == "quit-application-requested")
     aCancelQuit.data = true;
@@ -877,20 +795,19 @@ function Create3PaneGlobals()
 function loadStartFolder(initialUri)
 {
     var defaultServer = null;
+    var startFolder;
     var isLoginAtStartUpEnabled = false;
 
     //First get default account
-    if (!initialUri)
-    {
-        // Startup time.
-        defaultAccount = accountManager.defaultAccount;
+    if (initialUri) {
+        startFolder = MailUtils.getFolderForURI(initialUri);
+    } else {
+        var defaultAccount = accountManager.defaultAccount;
         if (defaultAccount) {
             defaultServer = defaultAccount.incomingServer;
+            var rootMsgFolder = defaultServer.rootMsgFolder;
 
-            // set the initialUri to the server, so we select it
-            // so we'll get account central
-            initialUri = defaultServer.serverURI;
-
+            startFolder = rootMsgFolder;
             // Enable check new mail once by turning checkmail pref 'on' to bring
             // all users to one plane. This allows all users to go to Inbox. User can
             // always go to server settings panel and turn off "Check for new mail at startup"
@@ -907,11 +824,10 @@ function loadStartFolder(initialUri)
             if (isLoginAtStartUpEnabled)
             {
                 //now find Inbox
-                var rootMsgFolder = defaultServer.rootMsgFolder;
                 const kInboxFlag = Ci.nsMsgFolderFlags.Inbox;
                 var inboxFolder = rootMsgFolder.getFolderWithFlags(kInboxFlag);
                 if (inboxFolder)
-                  initialUri = inboxFolder.URI;
+                  startFolder = inboxFolder;
             }
         } else {
             // If no default account then show account central page.
@@ -920,8 +836,15 @@ function loadStartFolder(initialUri)
 
    }
 
-   if (initialUri) {
-        SelectFolder(initialUri);
+   if (startFolder) {
+        try {
+          gFolderTreeView.selectFolder(startFolder);
+        } catch(ex) {
+          // This means we tried to select a folder that isn't in the current
+          // view. Just select the first one in the view then.
+          if (gFolderTreeView._rowMap.length)
+            gFolderTreeView.selectFolder(gFolderTreeView._rowMap[0]._folder);
+        }
 
         // Perform biff on the server to check for new mail, if:
         // the login at startup is enabled, and
@@ -962,79 +885,38 @@ function loadStartFolder(initialUri)
 
 function AddToSession()
 {
-  try {
-    var mailSession = Cc["@mozilla.org/messenger/services/session;1"]
-                        .getService(Ci.nsIMsgMailSession);
-    var nsIFolderListener = Ci.nsIFolderListener;
-    var notifyFlags = nsIFolderListener.intPropertyChanged | nsIFolderListener.event;
-    mailSession.AddFolderListener(folderListener, notifyFlags);
-  } catch (ex) {
-    dump("Error adding to session\n");
-  }
+  var nsIFolderListener = Ci.nsIFolderListener;
+  var notifyFlags = nsIFolderListener.intPropertyChanged |
+                    nsIFolderListener.event;
+  MailServices.mailSession.AddFolderListener(folderListener, notifyFlags);
 }
 
 function InitPanes()
 {
-  OnLoadFolderPane();
-  OnLoadLocationTree();
+  gFolderTreeView.load(document.getElementById("folderTree"),
+                       "folderTree.json");
+  var folderTree = document.getElementById("folderTree");
+  folderTree.addEventListener("click", FolderPaneOnClick, true);
+  folderTree.addEventListener("mousedown", TreeOnMouseDown, true);
+
   OnLoadThreadPane();
   SetupCommandUpdateHandlers();
 }
 
 function UnloadPanes()
 {
-  // Call to <mail3PaneWindowCommands.js>.
+  var folderTree = document.getElementById("folderTree");
+  folderTree.removeEventListener("click", FolderPaneOnClick, true);
+  folderTree.removeEventListener("mousedown", TreeOnMouseDown, true);
+  gFolderTreeView.unload("folderTree.json");
   UnloadCommandUpdateHandlers();
 }
 
-function InitializeDataSources()
-{
-  //Setup common mailwindow stuff.
-  AddDataSources();
-
-  SetupMoveCopyMenus('goMenu', accountManagerDataSource, folderDataSource);
-}
-
 function AddMutationObserver(callback)
 {
   new MutationObserver(callback).observe(callback(), {attributes: true, attributeFilter: ["hidden"]});
 }
 
-function UpdateFolderUnreadCol()
-{
-  var folderUnreadCol = document.getElementById("folderUnreadCol");
-  var folderNameCell = document.getElementById("folderNameCell");
-  var label = {true: "?folderTreeName", false: "?folderTreeSimpleName"};
-  folderNameCell.setAttribute("label", label[folderUnreadCol.hidden]);
-  return folderUnreadCol;
-}
-
-function UpgradeFolderPaneUI()
-{
-  // placeholder in case any new columns get added to the folder pane
-  // note that this function fails to notice a pane layout switch
-  // var folderPaneUIVersion = Services.prefs.getIntPref("mail.ui.folderpane.version");
-}
-
-function OnLoadFolderPane()
-{
-    UpgradeFolderPaneUI();
-    AddMutationObserver(UpdateFolderUnreadCol);
-
-    //Add folderDataSource and accountManagerDataSource to folderPane
-    var database = GetFolderDatasource();
-    database.AddDataSource(accountManagerDataSource);
-    database.AddDataSource(folderDataSource);
-
-    var folderTree = GetFolderTree();
-    folderTree.setAttribute("ref", "msgaccounts:/");
-
-    var folderTreeBuilder = folderTree.builder.QueryInterface(Ci.nsIXULTreeBuilder);
-    folderTreeBuilder.addObserver(folderObserver);
-    folderTree.addEventListener("click",FolderPaneOnClick,true);
-    folderTree.addEventListener("mousedown",TreeOnMouseDown,true);
-}
-
 function OnLoadThreadPane()
 {
   AddMutationObserver(UpdateAttachmentCol);
@@ -1049,63 +931,6 @@ function UpdateAttachmentCol()
   return attachmentCol;
 }
 
-function OnLoadLocationTree()
-{
-  var locationTree = document.getElementById("folderLocationPopup").tree;
-  if (locationTree)
-  {
-    locationTree.database.AddDataSource(accountManagerDataSource);
-    locationTree.database.AddDataSource(folderDataSource);
-    locationTree.setAttribute("ref", "msgaccounts:/");
-  }
-}
-
-function OnLocationTreeSelect(menulist)
-{
-  SelectFolder(menulist.getAttribute('uri'));
-}
-
-function UpdateLocationBar(resource)
-{
-    var tree = GetFolderTree();
-    var folders = document.getElementById('locationFolders');
-    var icon = document.getElementById('locationIcon');
-    var names = ['BiffState', 'NewMessages', 'HasUnreadMessages',
-        'SpecialFolder', 'IsServer', 'IsSecure', 'ServerType', 'NoSelect'];
-    let folder = resource.QueryInterface(Ci.nsIMsgFolder);
-    folders.setAttribute("label", folder.prettyName);
-
-    const rdfService = Cc["@mozilla.org/rdf/rdf-service;1"]
-                         .getService(Ci.nsIRDFService);
-    function GetFolderAttribute(tree, source, attribute) {
-      let property = rdfService.GetResource("http://home.netscape.com/NC-rdf#" + attribute);
-      let target = tree.database.GetTarget(source, property, true);
-      if (target)
-        target = target.QueryInterface(Ci.nsIRDFLiteral).Value;
-      return target;
-    }
-    for (var i in names) {
-        var name = names[i];
-        var value = GetFolderAttribute(tree, resource, name);
-        folders.setAttribute(name, value);
-        icon.setAttribute(name, value);
-    }
-    folders.setAttribute('uri', resource.Value);
-}
-
-function GetFolderDatasource()
-{
-  return GetFolderTree().database;
-}
-
-/* Functions for accessing particular parts of the window*/
-function GetFolderTree()
-{
-  if (!gFolderTree)
-    gFolderTree = document.getElementById("folderTree");
-  return gFolderTree;
-}
-
 function GetSearchInput()
 {
   if (!gSearchInput)
@@ -1224,19 +1049,28 @@ function TreeOnMouseDown(event)
     ChangeSelectionWithoutContentLoad(event, event.target.parentNode);
 }
 
+function FolderPaneContextMenuNewTab(event) {
+  var bgLoad = Services.prefs.getBoolPref("mail.tabs.loadInBackground");
+  if (event.shiftKey)
+    bgLoad = !bgLoad;
+  MsgOpenNewTabForFolder(bgLoad);
+}
+
 function FolderPaneOnClick(event)
 {
   // usually, we're only interested in tree content clicks, not scrollbars etc.
   if (event.originalTarget.localName != "treechildren")
     return;
 
+  var folderTree = document.getElementById("folderTree");
+
   // we may want to open the folder in a new tab on middle click
   if (event.button == kMouseButtonMiddle)
   {
     if (AllowOpenTabOnMiddleClick())
     {
-      MsgOpenNewTabForFolder();
-      RestoreSelectionWithoutContentLoad(GetFolderTree());
+      FolderPaneContextMenuNewTab(event);
+      RestoreSelectionWithoutContentLoad(folderTree);
       return;
     }
   }
@@ -1245,7 +1079,6 @@ function FolderPaneOnClick(event)
   if (event.button != kMouseButtonLeft)
     return;
 
-  var folderTree = GetFolderTree();
   var cell = folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY);
   if (cell.row == -1)
   {
@@ -1255,71 +1088,25 @@ function FolderPaneOnClick(event)
       event.stopPropagation();
     }
   }
-  else if ((event.detail == 2) &&
-           (cell.childElt != "twisty") &&
-           (folderTree.view.getLevel(cell.row) != 0))
-  {
-    FolderPaneDoubleClick(cell.row, event);
-  }
 }
 
-function FolderPaneDoubleClick(folderIndex, event)
-{
-  // In tabmail land, lazyness is dead!
-  // We either open the folder in a tab or a new window...
-  if (AllowOpenTabOnDoubleClick())
-  {
-    MsgOpenNewTabForFolder();
-  }
-  else
-  {
-    let folderResource = GetFolderResource(GetFolderTree(), folderIndex);
-    // Open a new msg window only if we are double clicking on
-    // folders or newsgroups.
-    MsgOpenNewWindowForFolder(folderResource.Value, nsMsgKey_None);
-  }
+function OpenMessageInNewTab(event) {
+  var bgLoad = Services.prefs.getBoolPref("mail.tabs.loadInBackground");
+  if (event.shiftKey)
+    bgLoad = !bgLoad;
 
-  // Double-clicking should not toggle the open/close state of the folder.
-  // This will happen if we don't prevent the event from bubbling to the
-  // default handler in tree.xml.
-  event.stopPropagation();
-}
-
-function ChangeSelection(tree, newIndex)
-{
-    if(newIndex >= 0)
-    {
-        tree.view.selection.select(newIndex);
-        tree.treeBoxObject.ensureRowIsVisible(newIndex);
-    }
+  MsgOpenNewTabForMessage(bgLoad);
 }
 
 function GetSelectedMsgFolders()
 {
-    var folderArray = [];
-    var folderTree = GetFolderTree();
-    var rangeCount = folderTree.view.selection.getRangeCount();
-
-    for (let i = 0; i < rangeCount; i++)
-    {
-        var startIndex = {};
-        var endIndex = {};
-        folderTree.view.selection.getRangeAt(i, startIndex, endIndex);
-        for (var j = startIndex.value; j <= endIndex.value; j++)
-        {
-          var folderResource = GetFolderResource(folderTree, j);
-          let msgFolder = folderResource.QueryInterface(Ci.nsIMsgFolder);
-          folderArray.push(msgFolder);
-        }
-    }
-
-    return folderArray;
+  return gFolderTreeView.getSelectedFolders();
 }
 
 function GetSelectedIndices(dbView)
 {
   try {
-    return dbView.getIndicesForSelection({});
+    return dbView.getIndicesForSelection();
   }
   catch (ex) {
     dump("ex = " + ex + "\n");
@@ -1391,28 +1178,26 @@ function SetNextMessageAfterDelete()
     gNextMessageViewIndexAfterDelete = treeSelection.currentIndex;
 }
 
-function EnsureFolderIndex(builder, msgFolder)
-{
-  // try to get the index of the folder in the tree
-  var index = builder.getIndexOfResource(msgFolder);
-  if (index == -1) {
-    // if we couldn't find the folder, open the parent
-    builder.toggleOpenState(EnsureFolderIndex(builder, msgFolder.parent));
-    index = builder.getIndexOfResource(msgFolder);
+function EnsureFolderIndex(treeView, msgFolder) {
+  // Try to get the index of the folder in the tree.
+  let index = treeView.getIndexOfFolder(msgFolder);
+  if (!index) {
+    // If we couldn't find the folder, open the parents.
+    let folder = msgFolder;
+    while (!index && folder) {
+      folder = folder.parent;
+      index = EnsureFolderIndex(treeView, folder);
+    }
+    if (index) {
+      treeView.toggleOpenState(index);
+      index = treeView.getIndexOfFolder(msgFolder);
+    }
   }
   return index;
 }
 
-function SelectFolder(folderUri)
-{
-  var folderTree = GetFolderTree();
-  let msgFolder = GetMsgFolderFromUri(folderUri);
-
-  // Before we can select a folder, we need to make sure it is "visible"
-  // in the tree. To do that, we need to ensure that all its
-  // ancestors are expanded.
-  var folderIndex = EnsureFolderIndex(folderTree.builderView, msgFolder);
-  ChangeSelection(folderTree, folderIndex);
+function SelectMsgFolder(msgFolder) {
+  gFolderTreeView.selectFolder(msgFolder);
 }
 
 function SelectMessage(messageUri)
@@ -1450,11 +1235,6 @@ function GetDBView()
   return gDBView;
 }
 
-function GetFolderResource(tree, index)
-{
-  return tree.builderView.getResourceAtIndex(index);
-}
-
 // Some of the per account junk mail settings have been
 // converted to global prefs. Let's try to migrate some
 // of those settings from the default account.
