@@ -1,8 +1,11 @@
-/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { AppConstants } =
+  ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
+
 var gLastMessageUriToLoad = null;
 var gThreadPaneCommandUpdater = null;
 
@@ -98,14 +101,7 @@ nsMsgDBViewCommandUpdater.prototype =
 
   summarizeSelection: function() {return false},
 
-  QueryInterface : function(iid)
-   {
-     if (iid.equals(Ci.nsIMsgDBViewCommandUpdater) ||
-         iid.equals(Ci.nsISupports))
-       return this;
-
-     throw Cr.NS_NOINTERFACE;
-    }
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIMsgDBViewCommandUpdater]),
 }
 
 function HandleColumnClick(columnID)
@@ -203,8 +199,14 @@ function ThreadPaneDoubleClick(event) {
 
 function ThreadPaneKeyPress(event)
 {
-  if (event.keyCode == KeyEvent.DOM_VK_RETURN)
-    ThreadPaneDoubleClick(event);
+  if (event.keyCode == KeyEvent.DOM_VK_RETURN) {
+    if ((AppConstants.platform == "macosx" ? event.metaKey : event.ctrlKey) &&
+        AllowOpenTabOnMiddleClick()) {
+      OpenMessageInNewTab(event);
+    } else {
+      ThreadPaneDoubleClick(event);
+    }
+  }
 }
 
 function MsgSortByThread()
